@@ -119,26 +119,6 @@ int      g_create_path(const char* path);
 int      g_remove_dir(const char* dirname);
 int      g_file_delete(const char* filename);
 int      g_file_get_size(const char* filename);
-int      g_strlen(const char* text);
-const char *g_strchr(const char *text, int c);
-char*    g_strcpy(char* dest, const char* src);
-char*    g_strncpy(char* dest, const char* src, int len);
-char*    g_strcat(char* dest, const char* src);
-char*    g_strdup(const char* in);
-char*    g_strndup(const char* in, const unsigned int maxlen);
-int      g_strcmp(const char* c1, const char* c2);
-int      g_strncmp(const char* c1, const char* c2, int len);
-int      g_strncmp_d(const char* c1, const char* c2, const char delim, int len);
-int      g_strcasecmp(const char* c1, const char* c2);
-int      g_strncasecmp(const char* c1, const char* c2, int len);
-int      g_atoi(const char* str);
-int      g_htoi(char* str);
-int      g_bytes_to_hexstr(const void *bytes, int num_bytes, char *out_str,
-                                  int bytes_out_str);
-int      g_pos(const char* str, const char* to_find);
-int      g_mbstowcs(twchar* dest, const char* src, int n);
-int      g_wcstombs(char* dest, const twchar* src, int n);
-int      g_strtrim(char* str, int trim_flags);
 long     g_load_library(char* in);
 int      g_free_library(long lib);
 void*    g_get_proc_address(long lib, const char* name);
@@ -161,6 +141,7 @@ int      g_getuid(void);
 int      g_getgid(void);
 int      g_setuid(int pid);
 int      g_setsid(void);
+int      g_getlogin(char *name, unsigned int len);
 int      g_setlogin(const char *name);
 int      g_waitchild(void);
 int      g_waitpid(int pid);
@@ -179,7 +160,6 @@ int      g_time2(void);
 int      g_time3(void);
 int      g_save_to_bmp(const char* filename, char* data, int stride_bytes,
                               int width, int height, int depth, int bits_per_pixel);
-int      g_text2bool(const char *s);
 void *   g_shmat(int shmid);
 int      g_shmdt(const void *shmaddr);
 int      g_gethostname(char *name, int len);
