@@ -32,6 +32,10 @@
  en-uk UK English 0x809
 */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,7 +44,7 @@
 #include <X11/XKBlib.h>
 #include <locale.h>
 
-extern int xfree86_to_evdev[137-8];
+extern int xfree86_to_evdev[137-8+1];
 
 int main(int argc, char **argv)
 {
@@ -132,7 +136,7 @@ int main(int argc, char **argv)
         fprintf(outf, "[%s]\n", sections[idx]);
         e.state = states[idx];
 
-        for (i = 8; i <= 137; i++) /* Keycodes */
+        for (i = 8; i < 137; i++) /* Keycodes */
         {
             if (is_evdev)
                 e.keycode = xfree86_to_evdev[i-8];
