@@ -23,30 +23,36 @@
 #endif
 
 #include "libxrdp.h"
+#include "string_calls.h"
 #include "xrdp_orders_rail.h"
 
 #include "ms-rdpbcgr.h"
 
-#define LOG_LEVEL 1
-#define LLOG(_level, _args) \
-    do { if (_level < LOG_LEVEL) { g_write _args ; } } while (0)
-#define LLOGLN(_level, _args) \
-    do { if (_level < LOG_LEVEL) { g_writeln _args ; } } while (0)
+
 
 #define MAX_BITMAP_BUF_SIZE (16 * 1024) /* 16K */
 
 /******************************************************************************/
 struct xrdp_session *EXPORT_CC
-libxrdp_init(tbus id, struct trans *trans)
+libxrdp_init(tbus id, struct trans *trans, const char *xrdp_ini)
 {
     struct xrdp_session *session;
 
     session = (struct xrdp_session *)g_malloc(sizeof(struct xrdp_session), 1);
     session->id = id;
     session->trans = trans;
+    if (xrdp_ini != NULL)
+    {
+        session->xrdp_ini = g_strdup(xrdp_ini);
+    }
+    else
+    {
+        session->xrdp_ini = g_strdup(XRDP_CFG_PATH "/xrdp.ini");
+    }
     session->rdp = xrdp_rdp_create(session, trans);
     session->orders = xrdp_orders_create(session, (struct xrdp_rdp *)session->rdp);
     session->client_info = &(((struct xrdp_rdp *)session->rdp)->client_info);
+    session->check_for_app_input = 1;
     return session;
 }
 
@@ -61,6 +67,7 @@ libxrdp_exit(struct xrdp_session *session)
 
     xrdp_orders_delete((struct xrdp_orders *)session->orders);
     xrdp_rdp_delete((struct xrdp_rdp *)session->rdp);
+    g_free(session->xrdp_ini);
     g_free(session);
     return 0;
 }
@@ -115,7 +122,7 @@ libxrdp_get_pdu_bytes(const char *aheader)
 /******************************************************************************/
 /* only used during connection */
 struct stream *
-libxrdp_force_read(struct trans* trans)
+libxrdp_force_read(struct trans *trans)
 {
     int bytes;
     struct stream *s;
@@ -125,24 +132,18 @@ libxrdp_force_read(struct trans* trans)
 
     if (trans_force_read(trans, 4) != 0)
     {
-        g_writeln("libxrdp_force_read: error");
+        LOG(LOG_LEVEL_WARNING, "libxrdp_force_read: header read error");
         return 0;
     }
     bytes = libxrdp_get_pdu_bytes(s->data);
-    if (bytes < 1)
+    if (bytes < 4 || bytes > s->size)
     {
-        g_writeln("libxrdp_force_read: error");
+        LOG(LOG_LEVEL_WARNING, "libxrdp_force_read: bad header length %d", bytes);
         return 0;
     }
-    if (bytes > 32 * 1024)
-    {
-        g_writeln("libxrdp_force_read: error");
-        return 0;
-    }
-
     if (trans_force_read(trans, bytes - 4) != 0)
     {
-        g_writeln("libxrdp_force_read: error");
+        LOG(LOG_LEVEL_WARNING, "libxrdp_force_read: Can't read PDU");
         return 0;
     }
     return s;
@@ -163,12 +164,12 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
     do_read = s == 0;
     if (do_read && session->up_and_running)
     {
-        g_writeln("libxrdp_process_data: error logic");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_process_data: error logic");
         return 1;
     }
     if (session->in_process_data != 0)
     {
-        g_writeln("libxrdp_process_data: error reentry");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_process_data: error reentry");
         return 1;
     }
     session->in_process_data++;
@@ -208,7 +209,7 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
             }
             if (s == 0)
             {
-                g_writeln("libxrdp_process_data: libxrdp_force_read failed");
+                LOG(LOG_LEVEL_ERROR, "libxrdp_process_data: libxrdp_force_read failed");
                 rv = 1;
                 break;
             }
@@ -216,12 +217,12 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
 
         if (xrdp_rdp_recv(rdp, s, &code) != 0)
         {
-            g_writeln("libxrdp_process_data: xrdp_rdp_recv failed");
+            LOG(LOG_LEVEL_ERROR, "libxrdp_process_data: xrdp_rdp_recv failed");
             rv = 1;
             break;
         }
 
-        DEBUG(("libxrdp_process_data code %d", code));
+        LOG_DEVEL(LOG_LEVEL_TRACE, "libxrdp_process_data code %d", code);
 
         switch (code)
         {
@@ -238,7 +239,7 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
             case PDUTYPE_DATAPDU:
                 if (xrdp_rdp_process_data(rdp, s) != 0)
                 {
-                    DEBUG(("libxrdp_process_data returned non zero"));
+                    LOG(LOG_LEVEL_ERROR, "libxrdp_process_data returned non zero");
                     cont = 0;
                     term = 1;
                 }
@@ -246,13 +247,13 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
             case 2: /* FASTPATH_INPUT_EVENT */
                 if (xrdp_fastpath_process_input_event(rdp->sec_layer->fastpath_layer, s) != 0)
                 {
-                     DEBUG(("libxrdp_process_data returned non zero"));
-                     cont = 0;
-                     term = 1;
+                    LOG(LOG_LEVEL_ERROR, "libxrdp_process_data returned non zero");
+                    cont = 0;
+                    term = 1;
                 }
                 break;
             default:
-                g_writeln("unknown in libxrdp_process_data: code= %d", code);
+                LOG(LOG_LEVEL_ERROR, "unknown in libxrdp_process_data: code= %d", code);
                 dead_lock_counter++;
                 break;
         }
@@ -261,8 +262,8 @@ libxrdp_process_data(struct xrdp_session *session, struct stream *s)
         {
             /*This situation can happen and this is a workaround*/
             cont = 0;
-            g_writeln("Serious programming error: we were locked in a deadly loop");
-            g_writeln("Remaining: %d", (int) (s->end - s->next_packet));
+            LOG(LOG_LEVEL_ERROR, "Serious programming error: we were locked in a deadly loop");
+            LOG(LOG_LEVEL_ERROR, "Remaining: %d", (int) (s->end - s->next_packet));
             s->next_packet = 0;
         }
 
@@ -291,7 +292,7 @@ libxrdp_send_palette(struct xrdp_session *session, int *palette)
         return 0;
     }
 
-    DEBUG(("libxrdp_send_palette sending palette"));
+    LOG_DEVEL(LOG_LEVEL_TRACE, "libxrdp_send_palette sending palette");
 
     /* clear orders */
     libxrdp_orders_force_send(session);
@@ -300,15 +301,16 @@ libxrdp_send_palette(struct xrdp_session *session, int *palette)
 
     if (session->client_info->use_fast_path & 1) /* fastpath output supported */
     {
-        LLOGLN(10, ("libxrdp_send_palette: fastpath"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_palette: fastpath");
         if (xrdp_rdp_init_fastpath((struct xrdp_rdp *)session->rdp, s) != 0)
         {
             free_stream(s);
             return 1;
         }
     }
-    else {
-        LLOGLN(10, ("libxrdp_send_palette: slowpath"));
+    else
+    {
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_palette: slowpath");
         xrdp_rdp_init_data((struct xrdp_rdp *)session->rdp, s);
     }
 
@@ -329,16 +331,16 @@ libxrdp_send_palette(struct xrdp_session *session, int *palette)
     s_mark_end(s);
     if (session->client_info->use_fast_path & 1) /* fastpath output supported */
     {
-       if (xrdp_rdp_send_fastpath((struct xrdp_rdp *)session->rdp, s,
+        if (xrdp_rdp_send_fastpath((struct xrdp_rdp *)session->rdp, s,
                                    FASTPATH_UPDATETYPE_PALETTE) != 0)
-       {
-           free_stream(s);
-           return 1;
-       }
+        {
+            free_stream(s);
+            return 1;
+        }
     }
     else
     {
-       xrdp_rdp_send_data((struct xrdp_rdp *)session->rdp, s,
+        xrdp_rdp_send_data((struct xrdp_rdp *)session->rdp, s,
                            RDP_DATA_PDU_UPDATE);
     }
     free_stream(s);
@@ -362,7 +364,7 @@ libxrdp_send_bell(struct xrdp_session *session)
 {
     struct stream *s = (struct stream *)NULL;
 
-    DEBUG(("libxrdp_send_bell sending bell signal"));
+    LOG_DEVEL(LOG_LEVEL_TRACE, "libxrdp_send_bell sending bell signal");
     /* see MS documentation: Server play sound PDU, TS_PLAY_SOUND_PDU_DATA */
 
     make_stream(s);
@@ -413,7 +415,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
     struct stream *temp_s = (struct stream *)NULL;
     tui32 pixel;
 
-    LLOGLN(10, ("libxrdp_send_bitmap: sending bitmap"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: sending bitmap");
     Bpp = (bpp + 7) / 8;
     e = (4 - width) & 3;
     switch (bpp)
@@ -433,14 +435,14 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
     line_bytes = width * Bpp;
     line_pad_bytes = line_bytes + e * Bpp;
 
-    LLOGLN(10, ("libxrdp_send_bitmap: bpp %d Bpp %d line_bytes %d "
-           "server_line_bytes %d", bpp, Bpp, line_bytes, server_line_bytes));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: bpp %d Bpp %d line_bytes %d "
+              "server_line_bytes %d", bpp, Bpp, line_bytes, server_line_bytes);
     make_stream(s);
     init_stream(s, MAX_BITMAP_BUF_SIZE);
 
     if (session->client_info->use_bitmap_comp)
     {
-        LLOGLN(10, ("libxrdp_send_bitmap: compression"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: compression");
         make_stream(temp_s);
         init_stream(temp_s, 65536);
         i = 0;
@@ -452,7 +454,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
 
         while (i > 0)
         {
-            LLOGLN(10, ("libxrdp_send_bitmap: i %d", i));
+            LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: i %d", i);
 
             total_bufsize = 0;
             num_updates = 0;
@@ -476,22 +478,22 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
 
                 if (bpp > 24)
                 {
-                    LLOGLN(10, ("libxrdp_send_bitmap: 32 bpp"));
+                    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: 32 bpp");
                     lines_sending = xrdp_bitmap32_compress(data, width, height,
                                                            s, 32,
-                                   (MAX_BITMAP_BUF_SIZE - 100) - total_bufsize,
+                                                           (MAX_BITMAP_BUF_SIZE - 100) - total_bufsize,
                                                            i - 1, temp_s, e, 0x10);
-                    LLOGLN(10, ("libxrdp_send_bitmap: i %d lines_sending %d",
-                           i, lines_sending));
+                    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: i %d lines_sending %d",
+                              i, lines_sending);
                 }
                 else
                 {
                     lines_sending = xrdp_bitmap_compress(data, width, height,
                                                          s, bpp,
-                                 (MAX_BITMAP_BUF_SIZE - 100) - total_bufsize,
+                                                         (MAX_BITMAP_BUF_SIZE - 100) - total_bufsize,
                                                          i - 1, temp_s, e);
-                    LLOGLN(10, ("libxrdp_send_bitmap: i %d lines_sending %d",
-                           i, lines_sending));
+                    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: i %d lines_sending %d",
+                              i, lines_sending);
                 }
 
                 if (lines_sending == 0)
@@ -534,29 +536,29 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
                     total_bufsize += 26; /* bytes since pop layer */
                 }
 
-                LLOGLN(10, ("libxrdp_send_bitmap: decompressed pixels %d "
-                       "decompressed bytes %d compressed bytes %d",
-                       lines_sending * (width + e),
-                       line_pad_bytes * lines_sending, bufsize));
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: decompressed pixels %d "
+                          "decompressed bytes %d compressed bytes %d",
+                          lines_sending * (width + e),
+                          line_pad_bytes * lines_sending, bufsize);
 
                 if (j > MAX_BITMAP_BUF_SIZE)
                 {
-                    LLOGLN(0, ("libxrdp_send_bitmap: error, decompressed "
-                           "size too big: %d bytes", j));
+                    LOG(LOG_LEVEL_WARNING, "libxrdp_send_bitmap: error, decompressed "
+                        "size too big: %d bytes", j);
                 }
 
                 if (bufsize > MAX_BITMAP_BUF_SIZE)
                 {
-                    LLOGLN(0, ("libxrdp_send_bitmap: error, compressed size "
-                           "too big: %d bytes", bufsize));
+                    LOG(LOG_LEVEL_WARNING, "libxrdp_send_bitmap: error, compressed size "
+                        "too big: %d bytes", bufsize);
                 }
 
                 s->p = s->end;
             }
             while (total_bufsize < MAX_BITMAP_BUF_SIZE && i > 0);
 
-            LLOGLN(10, ("libxrdp_send_bitmap: num_updates %d total_bufsize %d",
-                   num_updates, total_bufsize));
+            LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: num_updates %d total_bufsize %d",
+                      num_updates, total_bufsize);
 
             p_num_updates[0] = num_updates;
             p_num_updates[1] = num_updates >> 8;
@@ -565,8 +567,8 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
 
             if (total_bufsize > MAX_BITMAP_BUF_SIZE)
             {
-                LLOGLN(0, ("libxrdp_send_bitmap: error, total compressed "
-                       "size too big: %d bytes", total_bufsize));
+                LOG(LOG_LEVEL_WARNING, "libxrdp_send_bitmap: error, total compressed "
+                    "size too big: %d bytes", total_bufsize);
             }
         }
 
@@ -574,7 +576,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
     }
     else
     {
-        LLOGLN(10, ("libxrdp_send_bitmap: no compression"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_bitmap: no compression");
         total_lines = height;
         i = 0;
         p = data;
@@ -593,7 +595,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
 
                 if (lines_sending == 0)
                 {
-                    LLOGLN(0, ("libxrdp_send_bitmap: error, lines_sending == zero"));
+                    LOG(LOG_LEVEL_WARNING, "libxrdp_send_bitmap: error, lines_sending == zero");
                     break;
                 }
 
@@ -629,7 +631,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
                             q = q - server_line_bytes;
                             for (k = 0; k < width; k++)
                             {
-                                pixel = *((tui16*)(q + k * 2));
+                                pixel = *((tui16 *)(q + k * 2));
                                 out_uint16_le(s, pixel);
                             }
                             out_uint8s(s, e * 2);
@@ -641,7 +643,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
                             q = q - server_line_bytes;
                             for (k = 0; k < width; k++)
                             {
-                                pixel = *((tui32*)(q + k * 4));
+                                pixel = *((tui32 *)(q + k * 4));
                                 out_uint8(s, pixel);
                                 out_uint8(s, pixel >> 8);
                                 out_uint8(s, pixel >> 16);
@@ -655,7 +657,7 @@ libxrdp_send_bitmap(struct xrdp_session *session, int width, int height,
                             q = q - server_line_bytes;
                             for (k = 0; k < width; k++)
                             {
-                                pixel = *((int*)(q + k * 4));
+                                pixel = *((int *)(q + k * 4));
                                 out_uint32_le(s, pixel);
                             }
                             out_uint8s(s, e * 4);
@@ -688,7 +690,7 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
     int j;
     int data_bytes;
 
-    DEBUG(("libxrdp_send_pointer sending cursor"));
+    LOG_DEVEL(LOG_LEVEL_TRACE, "libxrdp_send_pointer sending cursor");
     if (bpp == 0)
     {
         bpp = 24;
@@ -698,14 +700,14 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
     {
         if (bpp != 24)
         {
-            g_writeln("libxrdp_send_pointer: error client does not support "
-                      "new cursors and bpp is %d", bpp);
+            LOG(LOG_LEVEL_ERROR, "libxrdp_send_pointer: error client does not support "
+                "new cursors and bpp is %d", bpp);
             return 1;
         }
     }
     if ((bpp == 15) && (bpp != 16) && (bpp != 24) && (bpp != 32))
     {
-        g_writeln("libxrdp_send_pointer: error");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_send_pointer: error");
         return 1;
     }
     make_stream(s);
@@ -713,7 +715,7 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
 
     if (session->client_info->use_fast_path & 1) /* fastpath output supported */
     {
-        LLOGLN(10, ("libxrdp_send_pointer: fastpath"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_pointer: fastpath");
         if (xrdp_rdp_init_fastpath((struct xrdp_rdp *)session->rdp, s) != 0)
         {
             free_stream(s);
@@ -732,21 +734,21 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
     }
     else /* slowpath */
     {
-        LLOGLN(10, ("libxrdp_send_pointer: slowpath"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_pointer: slowpath");
         xrdp_rdp_init_data((struct xrdp_rdp *)session->rdp, s);
         if ((session->client_info->pointer_flags & 1) == 0)
-         {
-             out_uint16_le(s, RDP_POINTER_COLOR);
-             out_uint16_le(s, 0); /* pad */
-             data_bytes = 3072;
-         }
-         else
-         {
-             out_uint16_le(s, RDP_POINTER_POINTER);
-             out_uint16_le(s, 0); /* pad */
-             out_uint16_le(s, bpp);
-             data_bytes = ((bpp + 7) / 8) * 32 * 32;
-         }
+        {
+            out_uint16_le(s, RDP_POINTER_COLOR);
+            out_uint16_le(s, 0); /* pad */
+            data_bytes = 3072;
+        }
+        else
+        {
+            out_uint16_le(s, RDP_POINTER_POINTER);
+            out_uint16_le(s, 0); /* pad */
+            out_uint16_le(s, bpp);
+            data_bytes = ((bpp + 7) / 8) * 32 * 32;
+        }
     }
 
 
@@ -808,7 +810,7 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
         if ((session->client_info->pointer_flags & 1) == 0)
         {
             if (xrdp_rdp_send_fastpath((struct xrdp_rdp *)session->rdp, s,
-                                        FASTPATH_UPDATETYPE_COLOR) != 0)
+                                       FASTPATH_UPDATETYPE_COLOR) != 0)
             {
                 free_stream(s);
                 return 1;
@@ -817,7 +819,7 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
         else
         {
             if (xrdp_rdp_send_fastpath((struct xrdp_rdp *)session->rdp, s,
-                                        FASTPATH_UPDATETYPE_POINTER) != 0)
+                                       FASTPATH_UPDATETYPE_POINTER) != 0)
             {
                 free_stream(s);
                 return 1;
@@ -827,7 +829,7 @@ libxrdp_send_pointer(struct xrdp_session *session, int cache_idx,
     else
     {
         xrdp_rdp_send_data((struct xrdp_rdp *)session->rdp, s,
-                          RDP_DATA_PDU_POINTER);
+                           RDP_DATA_PDU_POINTER);
     }
     free_stream(s);
     return 0;
@@ -839,14 +841,14 @@ libxrdp_set_pointer(struct xrdp_session *session, int cache_idx)
 {
     struct stream *s;
 
-    DEBUG(("libxrdp_set_pointer sending cursor index"));
+    LOG_DEVEL(LOG_LEVEL_TRACE, "libxrdp_set_pointer sending cursor index");
     make_stream(s);
     init_stream(s, 8192);
 
 
     if (session->client_info->use_fast_path & 1) /* fastpath output supported */
     {
-        LLOGLN(10, ("libxrdp_send_pointer: fastpath"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_pointer: fastpath");
         if (xrdp_rdp_init_fastpath((struct xrdp_rdp *)session->rdp, s) != 0)
         {
             free_stream(s);
@@ -855,7 +857,7 @@ libxrdp_set_pointer(struct xrdp_session *session, int cache_idx)
     }
     else
     {
-        LLOGLN(10, ("libxrdp_send_pointer: slowpath"));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_pointer: slowpath");
         xrdp_rdp_init_data((struct xrdp_rdp *)session->rdp, s);
         out_uint16_le(s, RDP_POINTER_CACHED);
         out_uint16_le(s, 0); /* pad */
@@ -867,7 +869,7 @@ libxrdp_set_pointer(struct xrdp_session *session, int cache_idx)
     if (session->client_info->use_fast_path & 1) /* fastpath output supported */
     {
         if (xrdp_rdp_send_fastpath((struct xrdp_rdp *)session->rdp, s,
-                                    FASTPATH_UPDATETYPE_CACHED) != 0)
+                                   FASTPATH_UPDATETYPE_CACHED) != 0)
         {
             free_stream(s);
             return 1;
@@ -876,7 +878,7 @@ libxrdp_set_pointer(struct xrdp_session *session, int cache_idx)
     else
     {
         xrdp_rdp_send_data((struct xrdp_rdp *)session->rdp, s,
-                            RDP_DATA_PDU_POINTER);
+                           RDP_DATA_PDU_POINTER);
     }
     free_stream(s);
     return 0;
@@ -971,21 +973,21 @@ libxrdp_orders_mem_blt(struct xrdp_session *session, int cache_id,
 
 /******************************************************************************/
 int
-libxrdp_orders_composite_blt(struct xrdp_session* session, int srcidx,
+libxrdp_orders_composite_blt(struct xrdp_session *session, int srcidx,
                              int srcformat, int srcwidth, int srcrepeat,
-                             int* srctransform, int mskflags,
+                             int *srctransform, int mskflags,
                              int mskidx, int mskformat, int mskwidth,
                              int mskrepeat, int op, int srcx, int srcy,
                              int mskx, int msky, int dstx, int dsty,
                              int width, int height, int dstformat,
-                             struct xrdp_rect* rect)
+                             struct xrdp_rect *rect)
 {
-  return xrdp_orders_composite_blt((struct xrdp_orders*)session->orders,
-                                   srcidx, srcformat, srcwidth, srcrepeat,
-                                   srctransform, mskflags,
-                                   mskidx, mskformat, mskwidth, mskrepeat,
-                                   op, srcx, srcy, mskx, msky, dstx, dsty,
-                                   width, height, dstformat, rect);
+    return xrdp_orders_composite_blt((struct xrdp_orders *)session->orders,
+                                     srcidx, srcformat, srcwidth, srcrepeat,
+                                     srctransform, mskflags,
+                                     mskidx, mskformat, mskwidth, mskrepeat,
+                                     op, srcx, srcy, mskx, msky, dstx, dsty,
+                                     width, height, dstformat, rect);
 }
 
 /******************************************************************************/
@@ -1049,29 +1051,36 @@ libxrdp_orders_send_font(struct xrdp_session *session,
 }
 
 /*****************************************************************************/
+/* Note : if this is called on a multimon setup, the client is resized
+ * to a single monitor */
 int EXPORT_CC
 libxrdp_reset(struct xrdp_session *session,
               int width, int height, int bpp)
 {
     if (session->client_info != 0)
     {
+        struct xrdp_client_info *client_info = session->client_info;
+
         /* older client can't resize */
-        if (session->client_info->build <= 419)
+        if (client_info->build <= 419)
         {
             return 0;
         }
 
-        /* if same, don't need to do anything */
-        if (session->client_info->width == width &&
-                session->client_info->height == height &&
-                session->client_info->bpp == bpp)
+        /* if same (and only one monitor on client) don't need to do anything */
+        if (client_info->width == width &&
+                client_info->height == height &&
+                client_info->bpp == bpp &&
+                (client_info->monitorCount == 0 || client_info->multimon == 0))
         {
             return 0;
         }
 
-        session->client_info->width = width;
-        session->client_info->height = height;
-        session->client_info->bpp = bpp;
+        client_info->width = width;
+        client_info->height = height;
+        client_info->bpp = bpp;
+        client_info->monitorCount = 0;
+        client_info->multimon = 0;
     }
     else
     {
@@ -1084,7 +1093,12 @@ libxrdp_reset(struct xrdp_session *session,
         return 1;
     }
 
-    /* shut down the rdp client */
+    /* shut down the rdp client
+     *
+     * When resetting the lib, disable application input checks, as
+     * otherwise we can send a channel message to the other end while
+     * the channels are inactive ([MS-RDPBCGR] 3.2.5.5.1 */
+    session->check_for_app_input = 0;
     if (xrdp_rdp_send_deactivate((struct xrdp_rdp *)session->rdp) != 0)
     {
         return 1;
@@ -1096,6 +1110,9 @@ libxrdp_reset(struct xrdp_session *session,
         return 1;
     }
 
+    /* Re-enable application input checks */
+    session->check_for_app_input = 1;
+
     return 0;
 }
 
@@ -1151,7 +1168,7 @@ libxrdp_query_channel(struct xrdp_session *session, int index,
 
     if (mcs->channel_list == NULL)
     {
-        g_writeln("libxrdp_query_channel - No channel initialized");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_query_channel - No channel initialized");
         return 1 ;
     }
 
@@ -1159,7 +1176,7 @@ libxrdp_query_channel(struct xrdp_session *session, int index,
 
     if (index < 0 || index >= count)
     {
-        DEBUG(("libxrdp_query_channel - Channel out of range %d", index));
+        LOG(LOG_LEVEL_ERROR, "libxrdp_query_channel - Channel out of range %d", index);
         return 1;
     }
 
@@ -1169,14 +1186,14 @@ libxrdp_query_channel(struct xrdp_session *session, int index,
     if (channel_item == 0)
     {
         /* this should not happen */
-        g_writeln("libxrdp_query_channel - channel item is 0");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_query_channel - channel item is 0");
         return 1;
     }
 
     if (channel_name != 0)
     {
         g_strncpy(channel_name, channel_item->name, 8);
-        DEBUG(("libxrdp_query_channel - Channel %d name %s", index, channel_name));
+        LOG(LOG_LEVEL_ERROR, "libxrdp_query_channel - Channel %d name %s", index, channel_name);
     }
 
     if (channel_flags != 0)
@@ -1204,7 +1221,7 @@ libxrdp_get_channel_id(struct xrdp_session *session, const char *name)
 
     if (mcs->channel_list == NULL)
     {
-        g_writeln("libxrdp_get_channel_id No channel initialized");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_get_channel_id No channel initialized");
         return -1 ;
     }
 
@@ -1256,7 +1273,7 @@ libxrdp_send_to_channel(struct xrdp_session *session, int channel_id,
 
     if (xrdp_channel_send(chan, s, channel_id, total_data_len, flags) != 0)
     {
-        g_writeln("libxrdp_send_to_channel: error, server channel data NOT sent to client channel");
+        LOG(LOG_LEVEL_ERROR, "libxrdp_send_to_channel: error, server channel data NOT sent to client channel");
         free_stream(s);
         return 1;
     }
@@ -1478,7 +1495,7 @@ libxrdp_codec_jpeg_compress(struct xrdp_session *session,
                             char *out_data, int *io_len)
 {
     struct xrdp_orders *orders;
-    void* jpeg_han;
+    void *jpeg_han;
 
     orders = (struct xrdp_orders *)(session->orders);
     jpeg_han = orders->jpeg_han;
@@ -1490,7 +1507,7 @@ libxrdp_codec_jpeg_compress(struct xrdp_session *session,
 /*****************************************************************************/
 int EXPORT_CC
 libxrdp_fastpath_send_surface(struct xrdp_session *session,
-                              char* data_pad, int pad_bytes,
+                              char *data_pad, int pad_bytes,
                               int data_bytes,
                               int destLeft, int destTop,
                               int destRight, int destBottom, int bpp,
@@ -1504,7 +1521,7 @@ libxrdp_fastpath_send_surface(struct xrdp_session *session,
     int max_bytes;
     int cmd_bytes;
 
-    LLOGLN(10, ("libxrdp_fastpath_send_surface:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_fastpath_send_surface:");
     if ((session->client_info->use_fast_path & 1) == 0)
     {
         return 1;
@@ -1563,7 +1580,7 @@ libxrdp_fastpath_send_frame_marker(struct xrdp_session *session,
     struct stream *s;
     struct xrdp_rdp *rdp;
 
-    LLOGLN(10, ("libxrdp_fastpath_send_frame_marker:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_fastpath_send_frame_marker:");
     if ((session->client_info->use_fast_path & 1) == 0)
     {
         return 1;
@@ -1597,7 +1614,7 @@ libxrdp_send_session_info(struct xrdp_session *session, const char *data,
 {
     struct xrdp_rdp *rdp;
 
-    LLOGLN(10, ("libxrdp_send_session_info:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "libxrdp_send_session_info:");
     rdp = (struct xrdp_rdp *) (session->rdp);
     return xrdp_rdp_send_session_info(rdp, data, data_bytes);
 }
