@@ -27,13 +27,7 @@
 #include "ms-rdpbcgr.h"
 #include "log.h"
 
-#define LOG_LEVEL 1
-#define LLOG(_level, _args) \
-    do { if (_level < LOG_LEVEL) { g_write _args ; } } while (0)
-#define LLOGLN(_level, _args) \
-    do { if (_level < LOG_LEVEL) { g_writeln _args ; } } while (0)
-#define LHEXDUMP(_level, _args) \
-    do { if (_level < LOG_LEVEL) { g_hexdump _args ; } } while (0)
+
 
 
 /*****************************************************************************/
@@ -42,11 +36,11 @@ xrdp_iso_create(struct xrdp_mcs *owner, struct trans *trans)
 {
     struct xrdp_iso *self;
 
-    LLOGLN(10, ("   in xrdp_iso_create"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   in xrdp_iso_create");
     self = (struct xrdp_iso *) g_malloc(sizeof(struct xrdp_iso), 1);
     self->mcs_layer = owner;
     self->trans = trans;
-    LLOGLN(10, ("   out xrdp_iso_create"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   out xrdp_iso_create");
     return self;
 }
 
@@ -80,11 +74,11 @@ xrdp_iso_negotiate_security(struct xrdp_iso *self)
             if (self->requestedProtocol & PROTOCOL_SSL)
             {
                 if (!g_file_readable(client_info->certificate) ||
-                    !g_file_readable(client_info->key_file))
+                        !g_file_readable(client_info->key_file))
                 {
                     /* certificate or privkey is not readable */
-                    log_message(LOG_LEVEL_DEBUG, "No readable certificates or "
-                                "private keys, cannot accept TLS connections");
+                    LOG(LOG_LEVEL_WARNING, "No readable certificates or "
+                        "private keys, cannot accept TLS connections");
                     self->failureCode = SSL_CERT_NOT_ON_SERVER;
                     rv = 1; /* error */
                 }
@@ -103,8 +97,8 @@ xrdp_iso_negotiate_security(struct xrdp_iso *self)
         case PROTOCOL_HYBRID_EX:
         default:
             if ((self->requestedProtocol & PROTOCOL_SSL) &&
-                g_file_readable(client_info->certificate) &&
-                g_file_readable(client_info->key_file))
+                    g_file_readable(client_info->certificate) &&
+                    g_file_readable(client_info->key_file))
             {
                 /* that's a patch since we don't support CredSSP for now */
                 self->selectedProtocol = PROTOCOL_SSL;
@@ -116,8 +110,8 @@ xrdp_iso_negotiate_security(struct xrdp_iso *self)
             break;
     }
 
-    log_message(LOG_LEVEL_DEBUG, "Security layer: requested %d, selected %d",
-                self->requestedProtocol, self->selectedProtocol);
+    LOG(LOG_LEVEL_DEBUG, "Security layer: requested %d, selected %d",
+        self->requestedProtocol, self->selectedProtocol);
     return rv;
 }
 
@@ -129,73 +123,90 @@ xrdp_iso_process_rdp_neg_req(struct xrdp_iso *self, struct stream *s)
     int flags;
     int len;
 
+    if (!s_check_rem(s, 7))
+    {
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_process_rdpNegReq: unexpected end-of-record");
+        return 1;
+    }
+
     in_uint8(s, flags);
     if (flags != 0x0 && flags != 0x8 && flags != 0x1)
     {
-        LLOGLN(10, ("xrdp_iso_process_rdpNegReq: error, flags: %x",flags));
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_process_rdpNegReq: error, flags: %x", flags);
         return 1;
     }
 
     in_uint16_le(s, len);
     if (len != 8)
     {
-        LLOGLN(10, ("xrdp_iso_process_rdpNegReq: error, length: %x",len));
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_process_rdpNegReq: error, length: %x", len);
         return 1;
     }
 
     in_uint32_le(s, self->requestedProtocol);
     if (self->requestedProtocol > 0xb)
     {
-        LLOGLN(10, ("xrdp_iso_process_rdpNegReq: error, requestedProtocol: %x",
-                self->requestedProtocol));
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_process_rdpNegReq: error, requestedProtocol: %x",
+            self->requestedProtocol);
         return 1;
     }
 
     return 0;
 }
-/*****************************************************************************/
-/* returns error */
+/*****************************************************************************
+ * Reads an X.224 PDU (X.224 section 13) preceded by a T.123 TPKT
+ * header (T.123 section 8)
+ *
+ * On entry, the TPKT header length field will have been inspected and used to
+ * set up the input stream.
+ *
+ * On exit, the TPKT header and the fixed part of the PDU header will have been
+ * removed from the stream.
+ *
+ * Returns error
+ *****************************************************************************/
 static int
 xrdp_iso_recv_msg(struct xrdp_iso *self, struct stream *s, int *code, int *len)
 {
     int ver;
-    int plen;
 
     *code = 0;
     *len = 0;
 
     if (s != self->trans->in_s)
     {
-        LLOGLN(10, ("xrdp_iso_recv_msg error logic"));
+        LOG(LOG_LEVEL_WARNING, "xrdp_iso_recv_msg error logic");
+    }
+
+    /* TPKT header is 4 bytes, then first 2 bytes of the X.224 CR-TPDU */
+    if (!s_check_rem(s, 6))
+    {
+        return 1;
     }
 
     in_uint8(s, ver);
-
-    if (ver != 3)
-    {
-        LLOGLN(10, ("xrdp_iso_recv_msg: bad ver"));
-        LHEXDUMP(10, (s->data, 4));
-        return 1;
-    }
-
-    in_uint8s(s, 1);
-    in_uint16_be(s, plen);
-
-    if (plen < 4)
-    {
-        return 1;
-    }
-
-    if (!s_check_rem(s, 2))
-    {
-        return 1;
-    }
-
+    in_uint8s(s, 3); /* Skip reserved field, plus length */
     in_uint8(s, *len);
     in_uint8(s, *code);
 
+    if (ver != 3)
+    {
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_recv_msg: bad ver");
+        LOG_DEVEL_HEXDUMP(LOG_LEVEL_ERROR, "header", s->data, 4);
+        return 1;
+    }
+
+    if (*len == 255)
+    {
+        /* X.224 13.2.1 - reserved value */
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_recv_msg: reserved length encountered");
+        LOG_DEVEL_HEXDUMP(LOG_LEVEL_ERROR, "header", s->data, 4);
+        return 1;
+    }
+
     if (*code == ISO_PDU_DT)
     {
+        /* Data PDU : X.224 13.7 */
         if (!s_check_rem(s, 1))
         {
             return 1;
@@ -204,6 +215,7 @@ xrdp_iso_recv_msg(struct xrdp_iso *self, struct stream *s, int *code, int *len)
     }
     else
     {
+        /* Other supported PDUs : X.224 13.x */
         if (!s_check_rem(s, 5))
         {
             return 1;
@@ -222,21 +234,21 @@ xrdp_iso_recv(struct xrdp_iso *self, struct stream *s)
     int code;
     int len;
 
-    LLOGLN(10, ("   in xrdp_iso_recv"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   in xrdp_iso_recv");
 
     if (xrdp_iso_recv_msg(self, s, &code, &len) != 0)
     {
-        LLOGLN(10, ("   out xrdp_iso_recv xrdp_iso_recv_msg return non zero"));
+        LOG(LOG_LEVEL_ERROR, "   out xrdp_iso_recv xrdp_iso_recv_msg return non zero");
         return 1;
     }
 
     if (code != ISO_PDU_DT || len != 2)
     {
-        LLOGLN(10, ("   out xrdp_iso_recv code != ISO_PDU_DT or length != 2"));
+        LOG(LOG_LEVEL_ERROR, "   out xrdp_iso_recv code != ISO_PDU_DT or length != 2");
         return 1;
     }
 
-    LLOGLN(10, ("   out xrdp_iso_recv"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   out xrdp_iso_recv");
     return 0;
 }
 /*****************************************************************************/
@@ -302,7 +314,20 @@ xrdp_iso_send_cc(struct xrdp_iso *self)
     free_stream(s);
     return 0;
 }
-/*****************************************************************************/
+/*****************************************************************************
+ * Process an X.224 connection request PDU
+ *
+ * See MS-RDPCGR v20190923 sections 2.2.1.1 and 3.3.5.3.1.
+ *
+ * From the latter, in particular:-
+ * - The length embedded in the TPKT header MUST be examined for
+ *   consistency with the received data. If there is a discrepancy, the
+ *   connection SHOULD be dropped
+ * - If the optional routingToken field exists it MUST be ignored.
+ * - If the optional cookie field is present it MUST be ignored.
+ * - If both the routingToken and cookie fields are present, the server
+ *   SHOULD continue with the connection.
+ *****************************************************************************/
 /* returns error */
 int
 xrdp_iso_incoming(struct xrdp_iso *self)
@@ -310,35 +335,47 @@ xrdp_iso_incoming(struct xrdp_iso *self)
     int rv = 0;
     int code;
     int len;
-    int cookie_index;
     int cc_type;
-    char text[256];
-    char *pend;
     struct stream *s;
+    int expected_pdu_len;
 
-    LLOGLN(10, ("   in xrdp_iso_incoming"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   in xrdp_iso_incoming");
 
     s = libxrdp_force_read(self->trans);
-    if (s == 0)
+    if (s == NULL)
     {
         return 1;
     }
 
     if (xrdp_iso_recv_msg(self, s, &code, &len) != 0)
     {
-        LLOGLN(0, ("xrdp_iso_incoming: xrdp_iso_recv_msg returned non zero"));
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_incoming: xrdp_iso_recv_msg returned non zero");
         return 1;
     }
 
-    if ((code != ISO_PDU_CR) || (len < 6))
+    if (code != ISO_PDU_CR)
     {
         return 1;
     }
 
+    /*
+     * Make sure the length indicator field extracted from the X.224
+     * connection request TPDU corresponds to the length in the TPKT header.
+     *
+     * We do this by seeing how the indicator field minus the counted
+     * octets in the TPDU header (6) compares with the space left in
+     * the stream.
+     */
+    expected_pdu_len = (s->end - s->p) + 6;
+    if (len != expected_pdu_len)
+    {
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_incoming: X.224 CR-TPDU length exp %d got %d",
+            expected_pdu_len, len);
+        return 1;
+    }
+
     /* process connection request */
-    pend = s->p + (len - 6);
-    cookie_index = 0;
-    while (s->p < pend)
+    while (s_check_rem(s, 1))
     {
         in_uint8(s, cc_type);
         switch (cc_type)
@@ -349,34 +386,36 @@ xrdp_iso_incoming(struct xrdp_iso *self)
                 self->rdpNegData = 1;
                 if (xrdp_iso_process_rdp_neg_req(self, s) != 0)
                 {
-                    LLOGLN(0, ("xrdp_iso_incoming: xrdp_iso_process_rdpNegReq returned non zero"));
+                    LOG(LOG_LEVEL_ERROR, "xrdp_iso_incoming: xrdp_iso_process_rdpNegReq returned non zero");
                     return 1;
                 }
                 break;
             case RDP_CORRELATION_INFO: /* rdpCorrelationInfo 6 */
                 // TODO
+                if (!s_check_rem(s, 1 + 2 + 16 + 16))
+                {
+                    LOG(LOG_LEVEL_ERROR, "xrdp_iso_incoming: short correlation info");
+                    return 1;
+                }
+
                 in_uint8s(s, 1 + 2 + 16 + 16);
                 break;
-            case 'C': /* Cookie routingToken */
-                while (s->p < pend)
+            case 'C': /* Cookie */
+                /* The routingToken and cookie fields are both ASCII
+                 * strings starting with the word 'Cookie: ' and
+                 * ending with CR+LF. We ignore both, so we do
+                 * not need to distinguish them  */
+                while (s_check_rem(s, 1))
                 {
-                    text[cookie_index] = cc_type;
-                    cookie_index++;
-                    if (cookie_index > 255)
-                    {
-                        cookie_index = 255;
-                    }
-                    if ((s->p[0] == 0x0D) && (s->p[1] == 0x0A))
-                    {
-                        in_uint8s(s, 2);
-                        text[cookie_index] = 0;
-                        cookie_index = 0;
-                        if (g_strlen(text) > 0)
-                        {
-                        }
-                        break;
-                    }
                     in_uint8(s, cc_type);
+                    if (cc_type == 0x0D && s_check_rem(s, 1))
+                    {
+                        in_uint8(s, cc_type);
+                        if (cc_type == 0x0A)
+                        {
+                            break;
+                        }
+                    }
                 }
                 break;
         }
@@ -388,11 +427,11 @@ xrdp_iso_incoming(struct xrdp_iso *self)
     /* send connection confirm back to client */
     if (xrdp_iso_send_cc(self) != 0)
     {
-        LLOGLN(0, ("xrdp_iso_incoming: xrdp_iso_send_cc returned non zero"));
+        LOG(LOG_LEVEL_ERROR, "xrdp_iso_incoming: xrdp_iso_send_cc returned non zero");
         return 1;
     }
 
-    LLOGLN(10, ("   out xrdp_iso_incoming"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   out xrdp_iso_incoming");
     return rv;
 }
 
@@ -413,7 +452,7 @@ xrdp_iso_send(struct xrdp_iso *self, struct stream *s)
 {
     int len;
 
-    LLOGLN(10, ("   in xrdp_iso_send"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   in xrdp_iso_send");
     s_pop_layer(s, iso_hdr);
     len = (int) (s->end - s->p);
     out_uint8(s, 3);
@@ -428,6 +467,6 @@ xrdp_iso_send(struct xrdp_iso *self, struct stream *s)
         return 1;
     }
 
-    LLOGLN(10, ("   out xrdp_iso_send"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "   out xrdp_iso_send");
     return 0;
 }
