@@ -24,6 +24,7 @@
 #include "defines.h"
 #include "xrdp_rail.h"
 #include "xrdp_client_info.h"
+#include "xrdp_constants.h"
 
 /* this is the freerdp main header */
 #include <freerdp/freerdp.h>
@@ -60,6 +61,8 @@ struct pointer_item
 
 #define CURRENT_MOD_VER 4
 
+struct source_info;
+
 struct mod
 {
     int size; /* size of this struct */
@@ -182,7 +185,7 @@ struct mod
     tintptr handle; /* pointer to self as long */
     tintptr wm;
     tintptr painter;
-    tintptr si;
+    struct source_info *si;
 
     /* mod data */
     int sck;
@@ -196,9 +199,9 @@ struct mod
     int vmaj;
     int vmin;
     int vrev;
-    char username[256];
-    char password[256];
-    char domain[256];
+    char username[INFO_CLIENT_MAX_CB_LEN];
+    char password[INFO_CLIENT_MAX_CB_LEN];
+    char domain[INFO_CLIENT_MAX_CB_LEN];
     int bool_keyBoardSynced ; /* Numlock can be out of sync, we hold state here to resolve */
     int keyBoardLockInfo ; /* Holds initial numlock capslock state */
 
