@@ -23,6 +23,7 @@
 
 #include "arch.h"
 #include "os_calls.h"
+#include "string_calls.h"
 #include "thread_calls.h"
 #include "trans.h"
 #include "chansrv.h"
@@ -36,9 +37,14 @@
 #include "rail.h"
 #include "xcommon.h"
 #include "chansrv_fuse.h"
+#include "chansrv_config.h"
 #include "xrdp_sockets.h"
 #include "audin.h"
 
+#include "ms-rdpbcgr.h"
+
+#define MAX_PATH 260
+
 static struct trans *g_lis_trans = 0;
 static struct trans *g_con_trans = 0;
 static struct trans *g_api_lis_trans = 0;
@@ -53,17 +59,16 @@ static int g_rail_index = -1;
 static tbus g_term_event = 0;
 static tbus g_thread_done_event = 0;
 
-static int g_use_unix_socket = 0;
+struct config_chansrv *g_cfg = NULL;
 
 int g_display_num = 0;
 int g_cliprdr_chan_id = -1; /* cliprdr */
 int g_rdpsnd_chan_id = -1;  /* rdpsnd  */
 int g_rdpdr_chan_id = -1;   /* rdpdr   */
 int g_rail_chan_id = -1;    /* rail    */
-int g_restrict_outbound_clipboard = 0;
 
 char *g_exec_name;
-tbus g_exec_event;
+tbus g_exec_event = 0;
 tbus g_exec_mutex;
 tbus g_exec_sem;
 int g_exec_pid = 0;
@@ -118,7 +123,7 @@ add_timeout(int msoffset, void (*callback)(void *data), void *data)
     struct timeout_obj *tobj;
     tui32 now;
 
-    LOG(10, ("add_timeout:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "add_timeout:");
     now = g_time3();
     tobj = g_new0(struct timeout_obj, 1);
     tobj->mstime = now + msoffset;
@@ -145,7 +150,7 @@ get_timeout(int *timeout)
     tui32 now;
     int ltimeout;
 
-    LOG(10, ("get_timeout:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "get_timeout:");
     ltimeout = *timeout;
     if (ltimeout < 1)
     {
@@ -157,7 +162,7 @@ get_timeout(int *timeout)
         now = g_time3();
         while (tobj != 0)
         {
-            LOG(10, ("  now %u tobj->mstime %u", now, tobj->mstime));
+            LOG_DEVEL(LOG_LEVEL_DEBUG, "  now %u tobj->mstime %u", now, tobj->mstime);
             if (now < tobj->mstime)
             {
                 ltimeout = tobj->mstime - now;
@@ -167,7 +172,7 @@ get_timeout(int *timeout)
     }
     if (ltimeout > 0)
     {
-        LOG(10, ("  ltimeout %d", ltimeout));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "  ltimeout %d", ltimeout);
         if (*timeout < 1)
         {
             *timeout = ltimeout;
@@ -193,7 +198,7 @@ check_timeout(void)
     int count;
     tui32 now;
 
-    LOG(10, ("check_timeout:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "check_timeout:");
     count = 0;
     tobj = g_timeout_head;
     if (tobj != 0)
@@ -233,7 +238,7 @@ check_timeout(void)
             }
         }
     }
-    LOG(10, ("  count %d", count));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "  count %d", count);
     return 0;
 }
 
@@ -259,8 +264,8 @@ send_channel_data(int chan_id, const char *data, int size)
     struct stream *s;
 
     if ((chan_id < 0) || (chan_id > 31) ||
-        (data == NULL) ||
-        (size < 1) || (size > MAX_CHANNEL_BYTES))
+            (data == NULL) ||
+            (size < 1) || (size > MAX_CHANNEL_BYTES))
     {
         /* bad param */
         return 1;
@@ -304,11 +309,11 @@ send_channel_data(int chan_id, const char *data, int size)
 /*****************************************************************************/
 /* returns error */
 int
-send_rail_drawing_orders(char* data, int size)
+send_rail_drawing_orders(char *data, int size)
 {
-    LOGM((LOG_LEVEL_DEBUG, "chansrv::send_rail_drawing_orders: size %d", size));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "chansrv::send_rail_drawing_orders: size %d", size);
 
-    struct stream* s;
+    struct stream *s;
     int error;
 
     s = trans_get_out_s(g_con_trans, 8192);
@@ -345,10 +350,10 @@ process_message_channel_setup(struct stream *s)
     g_rdpsnd_chan_id = -1;
     g_rdpdr_chan_id = -1;
     g_rail_chan_id = -1;
-    LOGM((LOG_LEVEL_DEBUG, "process_message_channel_setup:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_channel_setup:");
     in_uint16_le(s, num_chans);
-    LOGM((LOG_LEVEL_DEBUG, "process_message_channel_setup: num_chans %d",
-          num_chans));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_channel_setup: num_chans %d",
+              num_chans);
 
     for (index = 0; index < num_chans; index++)
     {
@@ -357,10 +362,7 @@ process_message_channel_setup(struct stream *s)
         in_uint8a(s, ci->name, 8);
         in_uint16_le(s, ci->id);
         in_uint16_le(s, ci->flags);
-        LOGM((LOG_LEVEL_DEBUG, "process_message_channel_setup: chan name '%s' "
-              "id %d flags %8.8x", ci->name, ci->id, ci->flags));
-
-        g_writeln("process_message_channel_setup: chan name '%s' "
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_channel_setup: chan name '%s' "
                   "id %d flags %8.8x", ci->name, ci->id, ci->flags);
 
         if (g_strcasecmp(ci->name, "cliprdr") == 0)
@@ -386,7 +388,7 @@ process_message_channel_setup(struct stream *s)
         }
         else
         {
-            LOG(10, ("other %s", ci->name));
+            LOG_DEVEL(LOG_LEVEL_DEBUG, "other %s", ci->name);
         }
 
         g_num_chan_items++;
@@ -441,9 +443,8 @@ process_message_channel_data(struct stream *s)
     in_uint16_le(s, chan_flags);
     in_uint16_le(s, length);
     in_uint32_le(s, total_length);
-    LOGM((LOG_LEVEL_DEBUG, "process_message_channel_data: chan_id %d "
-          "chan_flags %d", chan_id, chan_flags));
-    LOG(10, ("process_message_channel_data"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_channel_data: chan_id %d "
+              "chan_flags %d", chan_id, chan_flags);
     rv = 0;
 
     if (rv == 0)
@@ -496,7 +497,7 @@ process_message_channel_data(struct stream *s)
             }
             if (found == 0)
             {
-                LOG(0, ("process_message_channel_data: not found channel %d", chan_id));
+                LOG_DEVEL(LOG_LEVEL_INFO, "process_message_channel_data: not found channel %d", chan_id);
             }
         }
 
@@ -514,7 +515,7 @@ process_message_drdynvc_open_response(struct stream *s)
     int chan_id;
     int creation_status;
 
-    LOG(10, ("process_message_drdynvc_open_response:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_drdynvc_open_response:");
     if (!s_check_rem(s, 8))
     {
         return 1;
@@ -528,7 +529,7 @@ process_message_drdynvc_open_response(struct stream *s)
     drdynvc = g_drdynvcs + chan_id;
     if (drdynvc->status != CHANSRV_DRDYNVC_STATUS_OPEN_SENT)
     {
-        g_writeln("process_message_drdynvc_open_response: status not right");
+        LOG_DEVEL(LOG_LEVEL_ERROR, "process_message_drdynvc_open_response: status not right");
         return 1;
     }
     if (creation_status == 0)
@@ -558,7 +559,7 @@ process_message_drdynvc_close_response(struct stream *s)
     struct chansrv_drdynvc *drdynvc;
     int chan_id;
 
-    LOG(10, ("process_message_drdynvc_close_response:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_drdynvc_close_response:");
     if (!s_check_rem(s, 4))
     {
         return 1;
@@ -571,7 +572,7 @@ process_message_drdynvc_close_response(struct stream *s)
     drdynvc = g_drdynvcs + chan_id;
     if (drdynvc->status != CHANSRV_DRDYNVC_STATUS_CLOSE_SENT)
     {
-        g_writeln("process_message_drdynvc_close_response: status not right");
+        LOG_DEVEL(LOG_LEVEL_ERROR, "process_message_drdynvc_close_response: status not right");
         return 0;
     }
     drdynvc->status = CHANSRV_DRDYNVC_STATUS_CLOSED;
@@ -597,7 +598,7 @@ process_message_drdynvc_data_first(struct stream *s)
     int total_bytes;
     char *data;
 
-    LOG(10, ("process_message_drdynvc_data_first:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_drdynvc_data_first:");
     if (!s_check_rem(s, 12))
     {
         return 1;
@@ -636,7 +637,7 @@ process_message_drdynvc_data(struct stream *s)
     int bytes;
     char *data;
 
-    LOG(10, ("process_message_drdynvc_data:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "process_message_drdynvc_data:");
     if (!s_check_rem(s, 8))
     {
         return 1;
@@ -743,7 +744,7 @@ chansrv_drdynvc_data_first(int chan_id, const char *data, int data_bytes,
     struct stream *s;
     int error;
 
-    //g_writeln("chansrv_drdynvc_data_first: data_bytes %d total_data_bytes %d",
+    //LOG_DEVEL(LOG_LEVEL_DEBUG, "chansrv_drdynvc_data_first: data_bytes %d total_data_bytes %d",
     //          data_bytes, total_data_bytes);
     s = trans_get_out_s(g_con_trans, 8192);
     if (s == NULL)
@@ -770,7 +771,7 @@ chansrv_drdynvc_data(int chan_id, const char *data, int data_bytes)
     struct stream *s;
     int error;
 
-    //g_writeln("chansrv_drdynvc_data: data_bytes %d", data_bytes);
+    // LOG_DEVEL(LOG_LEVEL_DEBUG, "chansrv_drdynvc_data: data_bytes %d", data_bytes);
     s = trans_get_out_s(g_con_trans, 8192);
     if (s == NULL)
     {
@@ -794,7 +795,7 @@ chansrv_drdynvc_send_data(int chan_id, const char *data, int data_bytes)
 {
     int this_send_bytes;
 
-    //g_writeln("chansrv_drdynvc_send_data: data_bytes %d", data_bytes);
+    // LOG_DEVEL(LOG_LEVEL_DEBUG, "chansrv_drdynvc_send_data: data_bytes %d", data_bytes);
     if (data_bytes > 1590)
     {
         if (chansrv_drdynvc_data_first(chan_id, data, 1590, data_bytes) != 0)
@@ -870,7 +871,7 @@ process_message(void)
                 rv = process_message_drdynvc_data(s);
                 break;
             default:
-                LOGM((LOG_LEVEL_ERROR, "process_message: unknown msg %d", id));
+                LOG_DEVEL(LOG_LEVEL_ERROR, "process_message: unknown msg %d", id);
                 break;
         }
 
@@ -905,7 +906,7 @@ my_trans_data_in(struct trans *trans)
         return 1;
     }
 
-    LOGM((LOG_LEVEL_DEBUG, "my_trans_data_in:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "my_trans_data_in:");
     s = trans_get_in_s(trans);
     in_uint8s(s, 4); /* id */
     in_uint32_le(s, size);
@@ -1042,11 +1043,15 @@ my_api_trans_data_in(struct trans *trans)
     int rv;
     int bytes;
     int ver;
-    int channel_name_bytes;
     struct chansrv_drdynvc_procs procs;
-    char *chan_name;
+    /*
+     * Name is limited to CHANNEL_NAME_LEN for an SVC, or MAX_PATH
+     * bytes for a DVC
+     */
+    char chan_name[MAX(CHANNEL_NAME_LEN, MAX_PATH) + 1];
+    unsigned int channel_name_bytes;
 
-    //g_writeln("my_api_trans_data_in: extra_flags %d", trans->extra_flags);
+    //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: extra_flags %d", trans->extra_flags);
     rv = 0;
     ad = (struct xrdp_api_data *) (trans->callback_data);
     s = trans_get_in_s(trans);
@@ -1054,7 +1059,7 @@ my_api_trans_data_in(struct trans *trans)
     {
         in_uint32_le(s, bytes);
         in_uint32_le(s, ver);
-        //g_writeln("my_api_trans_data_in: bytes %d ver %d", bytes, ver);
+        //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: bytes %d ver %d", bytes, ver);
         if (ver != 0)
         {
             return 1;
@@ -1066,15 +1071,16 @@ my_api_trans_data_in(struct trans *trans)
     {
         rv = 1;
         in_uint32_le(s, channel_name_bytes);
-        //g_writeln("my_api_trans_data_in: channel_name_bytes %d", channel_name_bytes);
-        chan_name = g_new0(char, channel_name_bytes + 1);
-        if (chan_name == NULL)
+        //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: channel_name_bytes %d", channel_name_bytes);
+        if (channel_name_bytes > (sizeof(chan_name) - 1))
         {
             return 1;
         }
         in_uint8a(s, chan_name, channel_name_bytes);
+        chan_name[channel_name_bytes] = '\0';
+
         in_uint32_le(s, ad->chan_flags);
-        //g_writeln("my_api_trans_data_in: chan_name %s chan_flags 0x%8.8x", chan_name, ad->chan_flags);
+        //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: chan_name %s chan_flags 0x%8.8x", chan_name, ad->chan_flags);
         if (ad->chan_flags == 0)
         {
             /* SVC */
@@ -1128,11 +1134,10 @@ my_api_trans_data_in(struct trans *trans)
             procs.data = my_api_data;
             rv = chansrv_drdynvc_open(chan_name, ad->chan_flags,
                                       &procs, &(ad->chan_id));
-            //g_writeln("my_api_trans_data_in: chansrv_drdynvc_open rv %d "
+            //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: chansrv_drdynvc_open rv %d "
             //          "chan_id %d", rv, ad->chan_id);
             g_drdynvcs[ad->chan_id].xrdp_api_trans = trans;
         }
-        g_free(chan_name);
         init_stream(s, 0);
         trans->extra_flags = 2;
         trans->header_size = 0;
@@ -1142,7 +1147,7 @@ my_api_trans_data_in(struct trans *trans)
         bytes = g_sck_recv(trans->sck, s->data, s->size, 0);
         if (bytes < 1)
         {
-            //g_writeln("my_api_trans_data_in: disconnect");
+            //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: disconnect");
             return 1;
         }
         if (ad->chan_flags == 0)
@@ -1153,7 +1158,7 @@ my_api_trans_data_in(struct trans *trans)
         else
         {
             /* DVS */
-            //g_writeln("my_api_trans_data_in: s->size %d bytes %d", s->size, bytes);
+            //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_data_in: s->size %d bytes %d", s->size, bytes);
             rv = chansrv_drdynvc_send_data(ad->chan_id, s->data, bytes);
         }
         init_stream(s, 0);
@@ -1185,7 +1190,7 @@ my_trans_conn_in(struct trans *trans, struct trans *new_trans)
         return 1;
     }
 
-    LOGM((LOG_LEVEL_DEBUG, "my_trans_conn_in:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "my_trans_conn_in:");
     g_con_trans = new_trans;
     g_con_trans->trans_data_in = my_trans_data_in;
     g_con_trans->header_size = 8;
@@ -1204,10 +1209,10 @@ my_api_trans_conn_in(struct trans *trans, struct trans *new_trans)
 {
     struct xrdp_api_data *ad;
 
-    //g_writeln("my_api_trans_conn_in:");
+    //LOG_DEVEL(LOG_LEVEL_DEBUG, "my_api_trans_conn_in:");
     if ((trans == NULL) || (trans != g_api_lis_trans) || (new_trans == NULL))
     {
-        g_writeln("my_api_trans_conn_in: error");
+        LOG_DEVEL(LOG_LEVEL_ERROR, "my_api_trans_conn_in: error");
         return 1;
     }
     new_trans->trans_data_in = my_api_trans_data_in;
@@ -1216,7 +1221,7 @@ my_api_trans_conn_in(struct trans *trans, struct trans *new_trans)
     ad = g_new0(struct xrdp_api_data, 1);
     if (ad == NULL)
     {
-        g_writeln("my_api_trans_conn_in: error");
+        LOG_DEVEL(LOG_LEVEL_ERROR, "my_api_trans_conn_in: error");
         return 1;
     }
     new_trans->callback_data = ad;
@@ -1236,7 +1241,7 @@ setup_listen(void)
         trans_delete(g_lis_trans);
     }
 
-    if (g_use_unix_socket)
+    if (g_cfg->use_unix_socket)
     {
         g_lis_trans = trans_create(TRANS_MODE_UNIX, 8192, 8192);
         g_lis_trans->is_term = g_is_term;
@@ -1254,8 +1259,8 @@ setup_listen(void)
 
     if (error != 0)
     {
-        LOGM((LOG_LEVEL_ERROR, "setup_listen: trans_listen failed for port %s",
-              port));
+        LOG_DEVEL(LOG_LEVEL_ERROR, "setup_listen: trans_listen failed for port %s",
+                  port);
         return 1;
     }
 
@@ -1277,8 +1282,8 @@ setup_api_listen(void)
 
     if (error != 0)
     {
-        LOGM((LOG_LEVEL_ERROR, "setup_api_listen: trans_listen failed for port %s",
-              port));
+        LOG_DEVEL(LOG_LEVEL_ERROR, "setup_api_listen: trans_listen failed for port %s",
+                  port);
         return 1;
     }
 
@@ -1295,11 +1300,11 @@ api_con_trans_list_get_wait_objs_rw(intptr_t *robjs, int *rcount,
     struct trans *ltran;
 
     for (api_con_index = g_api_con_trans_list->count - 1;
-         api_con_index >= 0;
-         api_con_index--)
+            api_con_index >= 0;
+            api_con_index--)
     {
         ltran = (struct trans *)
-            list_get_item(g_api_con_trans_list, api_con_index);
+                list_get_item(g_api_con_trans_list, api_con_index);
         if (ltran != NULL)
         {
             trans_get_wait_objs_rw(ltran, robjs, rcount, wobjs, wcount,
@@ -1319,11 +1324,11 @@ api_con_trans_list_check_wait_objs(void)
     struct xrdp_api_data *ad;
 
     for (api_con_index = g_api_con_trans_list->count - 1;
-         api_con_index >= 0;
-         api_con_index--)
+            api_con_index >= 0;
+            api_con_index--)
     {
         ltran = (struct trans *)
-            list_get_item(g_api_con_trans_list, api_con_index);
+                list_get_item(g_api_con_trans_list, api_con_index);
         if (ltran != NULL)
         {
             if (trans_check_wait_objs(ltran) != 0)
@@ -1336,8 +1341,8 @@ api_con_trans_list_check_wait_objs(void)
                     chansrv_drdynvc_close(ad->chan_id);
                 }
                 for (drdynvc_index = 0;
-                     drdynvc_index < (int) ARRAYSIZE(g_drdynvcs);
-                     drdynvc_index++)
+                        drdynvc_index < (int) ARRAYSIZE(g_drdynvcs);
+                        drdynvc_index++)
                 {
                     if (g_drdynvcs[drdynvc_index].xrdp_api_trans == ltran)
                     {
@@ -1360,11 +1365,11 @@ api_con_trans_list_remove_all(void)
     struct trans *ltran;
 
     for (api_con_index = g_api_con_trans_list->count - 1;
-         api_con_index >= 0;
-         api_con_index--)
+            api_con_index >= 0;
+            api_con_index--)
     {
         ltran = (struct trans *)
-            list_get_item(g_api_con_trans_list, api_con_index);
+                list_get_item(g_api_con_trans_list, api_con_index);
         if (ltran != NULL)
         {
             list_remove_item(g_api_con_trans_list, api_con_index);
@@ -1387,7 +1392,7 @@ channel_thread_loop(void *in_val)
     int error;
     THREAD_RV rv;
 
-    LOGM((LOG_LEVEL_INFO, "channel_thread_loop: thread start"));
+    LOG_DEVEL(LOG_LEVEL_INFO, "channel_thread_loop: thread start");
     rv = 0;
     g_api_con_trans_list = list_create();
     setup_api_listen();
@@ -1409,7 +1414,7 @@ channel_thread_loop(void *in_val)
             check_timeout();
             if (g_is_wait_obj_set(g_term_event))
             {
-                LOGM((LOG_LEVEL_INFO, "channel_thread_loop: g_term_event set"));
+                LOG_DEVEL(LOG_LEVEL_INFO, "channel_thread_loop: g_term_event set");
                 clipboard_deinit();
                 sound_deinit();
                 devredir_deinit();
@@ -1421,8 +1426,8 @@ channel_thread_loop(void *in_val)
             {
                 if (trans_check_wait_objs(g_lis_trans) != 0)
                 {
-                    LOGM((LOG_LEVEL_INFO, "channel_thread_loop: "
-                          "trans_check_wait_objs error"));
+                    LOG_DEVEL(LOG_LEVEL_INFO, "channel_thread_loop: "
+                              "trans_check_wait_objs error");
                 }
             }
 
@@ -1430,8 +1435,8 @@ channel_thread_loop(void *in_val)
             {
                 if (trans_check_wait_objs(g_con_trans) != 0)
                 {
-                    LOGM((LOG_LEVEL_INFO, "channel_thread_loop: "
-                          "trans_check_wait_objs error resetting"));
+                    LOG_DEVEL(LOG_LEVEL_INFO, "channel_thread_loop: "
+                              "trans_check_wait_objs error resetting");
                     clipboard_deinit();
                     sound_deinit();
                     devredir_deinit();
@@ -1453,7 +1458,7 @@ channel_thread_loop(void *in_val)
             {
                 if (trans_check_wait_objs(g_api_lis_trans) != 0)
                 {
-                    LOG(0, ("channel_thread_loop: trans_check_wait_objs failed"));
+                    LOG_DEVEL(LOG_LEVEL_ERROR, "channel_thread_loop: trans_check_wait_objs failed");
                 }
             }
             /* check the wait_objs in g_api_con_trans_list */
@@ -1493,7 +1498,7 @@ channel_thread_loop(void *in_val)
     g_api_lis_trans = 0;
     api_con_trans_list_remove_all();
     list_delete(g_api_con_trans_list);
-    LOGM((LOG_LEVEL_INFO, "channel_thread_loop: thread stop"));
+    LOG_DEVEL(LOG_LEVEL_INFO, "channel_thread_loop: thread stop");
     g_set_wait_obj(g_thread_done_event);
     return rv;
 }
@@ -1502,7 +1507,7 @@ channel_thread_loop(void *in_val)
 void
 term_signal_handler(int sig)
 {
-    LOGM((LOG_LEVEL_INFO, "term_signal_handler: got signal %d", sig));
+    LOG_DEVEL(LOG_LEVEL_INFO, "term_signal_handler: got signal %d", sig);
     g_set_wait_obj(g_term_event);
 }
 
@@ -1510,7 +1515,7 @@ term_signal_handler(int sig)
 void
 nil_signal_handler(int sig)
 {
-    LOGM((LOG_LEVEL_INFO, "nil_signal_handler: got signal %d", sig));
+    LOG_DEVEL(LOG_LEVEL_INFO, "nil_signal_handler: got signal %d", sig);
 }
 
 /*****************************************************************************/
@@ -1519,14 +1524,14 @@ child_signal_handler(int sig)
 {
     int pid;
 
-    LOG(0, ("child_signal_handler:"));
+    LOG_DEVEL(LOG_LEVEL_INFO, "child_signal_handler:");
     do
     {
         pid = g_waitchild();
-        LOG(0, ("child_signal_handler: child pid %d", pid));
+        LOG_DEVEL(LOG_LEVEL_INFO, "child_signal_handler: child pid %d", pid);
         if ((pid == g_exec_pid) && (pid > 0))
         {
-            LOG(0, ("child_signal_handler: found pid %d", pid));
+            LOG_DEVEL(LOG_LEVEL_INFO, "child_signal_handler: found pid %d", pid);
             //shutdownx();
         }
     }
@@ -1537,7 +1542,7 @@ child_signal_handler(int sig)
 void
 segfault_signal_handler(int sig)
 {
-    LOG(0, ("segfault_signal_handler: entered......."));
+    LOG_DEVEL(LOG_LEVEL_ERROR, "segfault_signal_handler: entered.......");
     xfuse_deinit();
     exit(0);
 }
@@ -1546,7 +1551,7 @@ segfault_signal_handler(int sig)
 static void
 x_server_fatal_handler(void)
 {
-    LOGM((LOG_LEVEL_INFO, "xserver_fatal_handler: entered......."));
+    LOG_DEVEL(LOG_LEVEL_INFO, "xserver_fatal_handler: entered.......");
     /* At this point the X server has gone away. Dont make any X calls. */
     xfuse_deinit();
     exit(0);
@@ -1615,52 +1620,23 @@ get_display_num_from_display(char *display_text)
 int
 main_cleanup(void)
 {
-    g_delete_wait_obj(g_term_event);
-    g_delete_wait_obj(g_thread_done_event);
-    g_delete_wait_obj(g_exec_event);
-    tc_mutex_delete(g_exec_mutex);
-    g_deinit(); /* os_calls */
-    return 0;
-}
-
-/*****************************************************************************/
-static int
-read_ini(void)
-{
-    char filename[256];
-    struct list *names;
-    struct list *values;
-    char *name;
-    char *value;
-    int index;
-
-    g_memset(filename, 0, (sizeof(char) * 256));
-    names = list_create();
-    names->auto_free = 1;
-    values = list_create();
-    values->auto_free = 1;
-    g_use_unix_socket = 0;
-    g_snprintf(filename, 255, "%s/sesman.ini", XRDP_CFG_PATH);
-
-    if (file_by_name_read_section(filename, "Globals", names, values) == 0)
+    if (g_term_event != 0)
     {
-        for (index = 0; index < names->count; index++)
-        {
-            name = (char *)list_get_item(names, index);
-            value = (char *)list_get_item(values, index);
-
-            if (g_strcasecmp(name, "ListenAddress") == 0)
-            {
-                if (g_strcasecmp(value, "127.0.0.1") == 0)
-                {
-                    g_use_unix_socket = 1;
-                }
-            }
-        }
+        g_delete_wait_obj(g_term_event);
     }
-
-    list_delete(names);
-    list_delete(values);
+    if (g_thread_done_event != 0)
+    {
+        g_delete_wait_obj(g_thread_done_event);
+    }
+    if (g_exec_event != 0)
+    {
+        g_delete_wait_obj(g_exec_event);
+        tc_mutex_delete(g_exec_mutex);
+        tc_sem_delete(g_exec_sem);
+    }
+    log_end();
+    config_free(g_cfg);
+    g_deinit(); /* os_calls */
     return 0;
 }
 
@@ -1668,7 +1644,7 @@ read_ini(void)
 static int
 get_log_path(char *path, int bytes)
 {
-    char* log_path;
+    char *log_path;
     int rv;
 
     rv = 1;
@@ -1716,41 +1692,13 @@ get_log_path(char *path, int bytes)
     return rv;
 }
 
-/*****************************************************************************/
-static enum logLevels
-get_log_level(const char* level_str, enum logLevels default_level)
-{
-    static const char* levels[] = {
-        "LOG_LEVEL_ALWAYS",
-        "LOG_LEVEL_ERROR",
-        "LOG_LEVEL_WARNING",
-        "LOG_LEVEL_INFO",
-        "LOG_LEVEL_DEBUG",
-        "LOG_LEVEL_TRACE"
-    };
-    unsigned int i;
-
-    if (level_str == NULL || level_str[0] == 0)
-    {
-        return default_level;
-    }
-    for (i = 0; i < ARRAYSIZE(levels); ++i)
-    {
-        if (g_strcasecmp(levels[i], level_str) == 0)
-        {
-            return (enum logLevels) i;
-        }
-    }
-    return default_level;
-}
-
 /*****************************************************************************/
 static int
 run_exec(void)
 {
     int pid;
 
-    LOG(10, ("run_exec:"));
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "run_exec:");
     pid = g_fork();
 
     if (pid == 0)
@@ -1778,60 +1726,59 @@ main(int argc, char **argv)
     tbus waiters[4];
     int pid = 0;
     char text[256];
+    const char *config_path;
     char log_path[256];
     char *display_text;
     char log_file[256];
     enum logReturns error;
-    struct log_config logconfig;
-    enum logLevels log_level;
-    char *restrict_outbound_clipboard_env;
+    struct log_config *logconfig;
     g_init("xrdp-chansrv"); /* os_calls */
+    g_memset(g_drdynvcs, 0, sizeof(g_drdynvcs));
 
     log_path[255] = 0;
     if (get_log_path(log_path, 255) != 0)
     {
         g_writeln("error reading CHANSRV_LOG_PATH and HOME environment variable");
-        g_deinit();
+        main_cleanup();
         return 1;
     }
 
-    restrict_outbound_clipboard_env = g_getenv("CHANSRV_RESTRICT_OUTBOUND_CLIPBOARD");
-    if (restrict_outbound_clipboard_env != 0)
+    /*
+     * The user is unable at present to override the sysadmin-provided
+     * sesman.ini location */
+    config_path = XRDP_CFG_PATH "/sesman.ini";
+    if ((g_cfg = config_read(0, config_path)) == NULL)
     {
-        if (g_strcmp(restrict_outbound_clipboard_env, "1") == 0)
-        {
-            g_restrict_outbound_clipboard = 1;
-        }
+        main_cleanup();
+        return 1;
     }
+    config_dump(g_cfg);
 
-    read_ini();
     pid = g_getpid();
     display_text = g_getenv("DISPLAY");
-
-    if (display_text)
+    if (display_text != NULL)
+    {
         get_display_num_from_display(display_text);
-
-    log_level = get_log_level(g_getenv("CHANSRV_LOG_LEVEL"), LOG_LEVEL_INFO);
+    }
 
     /* starting logging subsystem */
-    g_memset(&logconfig, 0, sizeof(struct log_config));
-    logconfig.program_name = "xrdp-chansrv";
     g_snprintf(log_file, 255, "%s/xrdp-chansrv.%d.log", log_path, g_display_num);
     g_writeln("chansrv::main: using log file [%s]", log_file);
-
     if (g_file_exist(log_file))
     {
         g_file_delete(log_file);
     }
 
-    g_memset(g_drdynvcs, 0, sizeof(g_drdynvcs));
-
-    logconfig.log_file = log_file;
-    logconfig.fd = -1;
-    logconfig.log_level = log_level;
-    logconfig.enable_syslog = 0;
-    logconfig.syslog_level = LOG_LEVEL_ALWAYS;
-    error = log_start_from_param(&logconfig);
+    logconfig = log_config_init_from_config(config_path, "xrdp-chansrv", "Chansrv");
+    if (logconfig->log_file != NULL)
+    {
+        g_free(logconfig->log_file);
+    }
+    logconfig->log_file = log_file;
+    error = log_start_from_param(logconfig);
+    logconfig->log_file = NULL;
+    log_config_free(logconfig);
+    logconfig = NULL;
 
     if (error != LOG_STARTUP_OK)
     {
@@ -1849,11 +1796,11 @@ main(int argc, char **argv)
                 break;
         }
 
-        g_deinit();
+        main_cleanup();
         return 1;
     }
 
-    LOGM((LOG_LEVEL_ALWAYS, "main: app started pid %d(0x%8.8x)", pid, pid));
+    LOG_DEVEL(LOG_LEVEL_INFO, "main: app started pid %d(0x%8.8x)", pid, pid);
     /*  set up signal handler  */
     g_signal_terminate(term_signal_handler); /* SIGTERM */
     g_signal_user_interrupt(term_signal_handler); /* SIGINT */
@@ -1864,16 +1811,16 @@ main(int argc, char **argv)
     /* Cater for the X server exiting unexpectedly */
     xcommon_set_x_server_fatal_handler(x_server_fatal_handler);
 
-    LOGM((LOG_LEVEL_INFO, "main: DISPLAY env var set to %s", display_text));
+    LOG_DEVEL(LOG_LEVEL_INFO, "main: DISPLAY env var set to %s", display_text);
 
     if (g_display_num == 0)
     {
-        LOGM((LOG_LEVEL_ERROR, "main: error, display is zero"));
-        g_deinit();
+        LOG_DEVEL(LOG_LEVEL_ERROR, "main: error, display is zero");
+        main_cleanup();
         return 1;
     }
 
-    LOGM((LOG_LEVEL_INFO, "main: using DISPLAY %d", g_display_num));
+    LOG_DEVEL(LOG_LEVEL_INFO, "main: using DISPLAY %d", g_display_num);
     g_snprintf(text, 255, "xrdp_chansrv_%8.8x_main_term", pid);
     g_term_event = g_create_wait_obj(text);
     g_snprintf(text, 255, "xrdp_chansrv_%8.8x_thread_done", pid);
@@ -1891,7 +1838,7 @@ main(int argc, char **argv)
 
         if (g_obj_wait(waiters, 2, 0, 0, 0) != 0)
         {
-            LOGM((LOG_LEVEL_ERROR, "main: error, g_obj_wait failed"));
+            LOG_DEVEL(LOG_LEVEL_ERROR, "main: error, g_obj_wait failed");
             break;
         }
 
@@ -1912,15 +1859,14 @@ main(int argc, char **argv)
         /* wait for thread to exit */
         if (g_obj_wait(&g_thread_done_event, 1, 0, 0, 0) != 0)
         {
-            LOGM((LOG_LEVEL_ERROR, "main: error, g_obj_wait failed"));
+            LOG_DEVEL(LOG_LEVEL_ERROR, "main: error, g_obj_wait failed");
             break;
         }
     }
 
     /* cleanup */
     main_cleanup();
-    LOGM((LOG_LEVEL_INFO, "main: app exiting pid %d(0x%8.8x)", pid, pid));
-    g_deinit();
+    LOG_DEVEL(LOG_LEVEL_INFO, "main: app exiting pid %d(0x%8.8x)", pid, pid);
     return 0;
 }
 
