@@ -24,7 +24,12 @@
  *
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "libscp_session.h"
+#include "string_calls.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -34,7 +39,7 @@
 
 /*******************************************************************/
 struct SCP_SESSION *
-scp_session_create()
+scp_session_create(void)
 {
     struct SCP_SESSION *s;
 
@@ -42,7 +47,7 @@ scp_session_create()
 
     if (0 == s)
     {
-        log_message(LOG_LEVEL_ERROR, "[session:%d] session create: malloc error", __LINE__);
+        LOG(LOG_LEVEL_ERROR, "[session:%d] session create: malloc error", __LINE__);
         return 0;
     }
 
@@ -77,14 +82,14 @@ scp_session_set_type(struct SCP_SESSION *s, tui8 type)
 
             if (NULL == s->mng)
             {
-                log_message(LOG_LEVEL_ERROR, "[session:%d] set_type: internal error", __LINE__);
+                LOG(LOG_LEVEL_ERROR, "[session:%d] set_type: internal error", __LINE__);
                 return 1;
             }
 
             break;
 
         default:
-            log_message(LOG_LEVEL_WARNING, "[session:%d] set_type: unknown type", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[session:%d] set_type: unknown type", __LINE__);
             return 1;
     }
 
@@ -104,7 +109,7 @@ scp_session_set_version(struct SCP_SESSION *s, tui32 version)
             s->version = 1;
             break;
         default:
-            log_message(LOG_LEVEL_WARNING, "[session:%d] set_version: unknown version", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[session:%d] set_version: unknown version", __LINE__);
             return 1;
     }
 
@@ -168,7 +173,7 @@ scp_session_set_locale(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_locale: null locale", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_locale: null locale", __LINE__);
         s->locale[0] = '\0';
         return 1;
     }
@@ -184,7 +189,7 @@ scp_session_set_username(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_username: null username", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_username: null username", __LINE__);
         return 1;
     }
 
@@ -197,7 +202,7 @@ scp_session_set_username(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->username)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_username: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_username: strdup error", __LINE__);
         return 1;
     }
 
@@ -210,7 +215,7 @@ scp_session_set_password(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_password: null password", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_password: null password", __LINE__);
         return 1;
     }
 
@@ -223,7 +228,7 @@ scp_session_set_password(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->password)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_password: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_password: strdup error", __LINE__);
         return 1;
     }
 
@@ -236,7 +241,7 @@ scp_session_set_domain(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_domain: null domain", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_domain: null domain", __LINE__);
         return 1;
     }
 
@@ -249,7 +254,7 @@ scp_session_set_domain(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->domain)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_domain: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_domain: strdup error", __LINE__);
         return 1;
     }
 
@@ -262,7 +267,7 @@ scp_session_set_program(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_program: null program", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_program: null program", __LINE__);
         return 1;
     }
 
@@ -275,7 +280,7 @@ scp_session_set_program(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->program)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_program: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_program: strdup error", __LINE__);
         return 1;
     }
 
@@ -288,7 +293,7 @@ scp_session_set_directory(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_directory: null directory", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_directory: null directory", __LINE__);
         return 1;
     }
 
@@ -301,7 +306,7 @@ scp_session_set_directory(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->directory)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_directory: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_directory: strdup error", __LINE__);
         return 1;
     }
 
@@ -314,7 +319,7 @@ scp_session_set_client_ip(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_client_ip: null ip", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_client_ip: null ip", __LINE__);
         return 1;
     }
 
@@ -327,7 +332,7 @@ scp_session_set_client_ip(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->client_ip)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_client_ip: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_client_ip: strdup error", __LINE__);
         return 1;
     }
 
@@ -340,7 +345,7 @@ scp_session_set_hostname(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_hostname: null hostname", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_hostname: null hostname", __LINE__);
         return 1;
     }
 
@@ -353,7 +358,7 @@ scp_session_set_hostname(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->hostname)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_hostname: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_hostname: strdup error", __LINE__);
         return 1;
     }
 
@@ -366,7 +371,7 @@ scp_session_set_errstr(struct SCP_SESSION *s, const char *str)
 {
     if (0 == str)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_errstr: null string", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_errstr: null string", __LINE__);
         return 1;
     }
 
@@ -379,7 +384,7 @@ scp_session_set_errstr(struct SCP_SESSION *s, const char *str)
 
     if (0 == s->errstr)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_errstr: strdup error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_errstr: strdup error", __LINE__);
         return 1;
     }
 
@@ -421,7 +426,7 @@ scp_session_set_guid(struct SCP_SESSION *s, const tui8 *guid)
 {
     if (0 == guid)
     {
-        log_message(LOG_LEVEL_WARNING, "[session:%d] set_guid: null guid", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[session:%d] set_guid: null guid", __LINE__);
         return 1;
     }
 
