@@ -29,6 +29,7 @@
 #endif
 
 #include "libscp_v1c_mng.h"
+#include "string_calls.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -49,8 +50,8 @@ scp_v1c_mng_connect(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     init_stream(c->out_s, c->out_s->size);
     init_stream(c->in_s, c->in_s->size);
 
-    size = 12 + 4 + g_strlen(s->hostname) + g_strlen(s->username) +
-           g_strlen(s->password);
+    size = (12 + 4 + g_strlen(s->hostname) + g_strlen(s->username) +
+            g_strlen(s->password));
 
     if (s->addr_type == SCP_ADDRESS_TYPE_IPV4)
     {
@@ -96,7 +97,7 @@ scp_v1c_mng_connect(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
     if (0 != scp_tcp_force_send(c->in_sck, c->out_s->data, size))
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
         return SCP_CLIENT_STATE_NETWORK_ERR;
     }
 
@@ -132,7 +133,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
     if (0 != scp_tcp_force_send(c->in_sck, c->out_s->data, size))
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
         return SCP_CLIENT_STATE_NETWORK_ERR;
     }
 
@@ -143,7 +144,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (0 != scp_tcp_force_recv(c->in_sck, c->in_s->data, 8))
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_NETWORK_ERR;
         }
@@ -152,7 +153,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (version != 1)
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: version error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: version error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_VERSION_ERR;
         }
@@ -161,7 +162,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (size < 12)
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: size error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: size error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_SIZE_ERR;
         }
@@ -170,7 +171,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (0 != scp_tcp_force_recv(c->in_sck, c->in_s->data, size - 8))
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_NETWORK_ERR;
         }
@@ -179,7 +180,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (cmd != SCP_COMMAND_SET_MANAGE)
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_SEQUENCE_ERR;
         }
@@ -188,7 +189,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
         if (cmd != SCP_CMD_MNG_LIST) /* session list */
         {
-            log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
+            LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
             g_free(ds);
             return SCP_CLIENT_STATE_SEQUENCE_ERR;
         }
@@ -205,7 +206,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
                 (*scount) = sescnt;
                 (*s) = NULL;
 
-                LOG_DBG("[v1c_mng] end list - no session on TS");
+                LOG_DEVEL(LOG_LEVEL_DEBUG, "[v1c_mng] end list - no session on TS");
                 return SCP_CLIENT_STATE_LIST_OK;
             }
 
@@ -213,7 +214,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
 
             if (ds == 0)
             {
-                log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: internal error", __LINE__);
+                LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: internal error", __LINE__);
                 return SCP_CLIENT_STATE_INTERNAL_ERR;
             }
         }
@@ -262,7 +263,7 @@ scp_v1c_mng_get_session_list(struct SCP_CONNECTION *c, int *scount,
     (*scount) = sescnt;
     (*s) = ds;
 
-    LOG_DBG("[v1c_mng] end list");
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "[v1c_mng] end list");
     return SCP_CLIENT_STATE_LIST_OK;
 }
 
@@ -373,7 +374,7 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
     if (0 != scp_tcp_force_recv(c->in_sck, c->in_s->data, 8))
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
         return SCP_CLIENT_STATE_NETWORK_ERR;
     }
 
@@ -381,7 +382,7 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
     if (version != 1)
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: version error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: version error", __LINE__);
         return SCP_CLIENT_STATE_VERSION_ERR;
     }
 
@@ -392,7 +393,7 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     /* read the rest of the packet */
     if (0 != scp_tcp_force_recv(c->in_sck, c->in_s->data, size - 8))
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: network error", __LINE__);
         return SCP_CLIENT_STATE_NETWORK_ERR;
     }
 
@@ -400,7 +401,7 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
     if (cmd != SCP_COMMAND_SET_MANAGE)
     {
-        log_message(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
+        LOG(LOG_LEVEL_WARNING, "[v1c_mng:%d] connection aborted: sequence error", __LINE__);
         return SCP_CLIENT_STATE_SEQUENCE_ERR;
     }
 
@@ -408,7 +409,7 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
     if (cmd == SCP_CMD_MNG_LOGIN_ALLOW) /* connection ok */
     {
-        log_message(LOG_LEVEL_INFO, "[v1c_mng:%d] connection ok", __LINE__);
+        LOG(LOG_LEVEL_INFO, "[v1c_mng:%d] connection ok", __LINE__);
         return SCP_CLIENT_STATE_OK;
     }
     else if (cmd == SCP_CMD_MNG_LOGIN_DENY) /* connection denied */
@@ -418,10 +419,10 @@ _scp_v1c_mng_check_response(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
         in_uint8a(c->in_s, buf, dim);
         scp_session_set_errstr(s, buf);
 
-        log_message(LOG_LEVEL_INFO, "[v1c_mng:%d] connection denied: %s", __LINE__ , s->errstr);
+        LOG(LOG_LEVEL_INFO, "[v1c_mng:%d] connection denied: %s", __LINE__, s->errstr);
         return SCP_CLIENT_STATE_CONNECTION_DENIED;
     }
 
-    log_message(LOG_LEVEL_WARNING, "[v1c-mng:%d] connection aborted: sequence error", __LINE__);
+    LOG(LOG_LEVEL_WARNING, "[v1c-mng:%d] connection aborted: sequence error", __LINE__);
     return SCP_CLIENT_STATE_SEQUENCE_ERR;
 }
