@@ -54,23 +54,23 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
             {
                 /* the user is member of the correct groups. */
                 scp_v0s_replyauthentication(c, errorcode);
-                log_message(LOG_LEVEL_INFO, "Access permitted for user: %s",
-                            s->username);
+                LOG(LOG_LEVEL_INFO, "Access permitted for user: %s",
+                    s->username);
                 /* g_writeln("Connection allowed"); */
             }
             else
             {
                 scp_v0s_replyauthentication(c, 32 + 3); /* all first 32 are reserved for PAM errors */
-                log_message(LOG_LEVEL_INFO, "Username okey but group problem for "
-                            "user: %s", s->username);
+                LOG(LOG_LEVEL_INFO, "Username okey but group problem for "
+                    "user: %s", s->username);
                 /* g_writeln("user password ok, but group problem"); */
             }
         }
         else
         {
             /* g_writeln("username or password error"); */
-            log_message(LOG_LEVEL_INFO, "Username or password error for user: %s",
-                        s->username);
+            LOG(LOG_LEVEL_INFO, "Username or password error for user: %s",
+                s->username);
             scp_v0s_replyauthentication(c, errorcode);
         }
     }
@@ -85,55 +85,55 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
             g_memcpy(s->guid, s_item->guid, 16);
             if (0 != s->client_ip)
             {
-                log_message( LOG_LEVEL_INFO, "++ reconnected session: username %s, "
-                             "display :%d.0, session_pid %d, ip %s",
-                             s->username, display, s_item->pid, s->client_ip);
+                LOG( LOG_LEVEL_INFO, "++ reconnected session: username %s, "
+                     "display :%d.0, session_pid %d, ip %s",
+                     s->username, display, s_item->pid, s->client_ip);
             }
             else
             {
-                log_message(LOG_LEVEL_INFO, "++ reconnected session: username %s, "
-                            "display :%d.0, session_pid %d", s->username, display,
-                            s_item->pid);
+                LOG(LOG_LEVEL_INFO, "++ reconnected session: username %s, "
+                    "display :%d.0, session_pid %d", s->username, display,
+                    s_item->pid);
             }
 
             session_reconnect(display, s->username, data);
         }
         else
         {
-            LOG_DBG("pre auth");
+            LOG_DEVEL(LOG_LEVEL_DEBUG, "pre auth");
 
             if (1 == access_login_allowed(s->username))
             {
                 tui8 guid[16];
 
-                g_random((char*)guid, 16);
+                g_random((char *)guid, 16);
                 scp_session_set_guid(s, guid);
 
                 if (0 != s->client_ip)
                 {
-                    log_message(LOG_LEVEL_INFO, "++ created session (access granted): "
-                                "username %s, ip %s", s->username, s->client_ip);
+                    LOG(LOG_LEVEL_INFO, "++ created session (access granted): "
+                        "username %s, ip %s", s->username, s->client_ip);
                 }
                 else
                 {
-                    log_message(LOG_LEVEL_INFO, "++ created session (access granted): "
-                                "username %s", s->username);
+                    LOG(LOG_LEVEL_INFO, "++ created session (access granted): "
+                        "username %s", s->username);
                 }
 
                 if (SCP_SESSION_TYPE_XVNC == s->type)
                 {
-                    log_message( LOG_LEVEL_INFO, "starting Xvnc session...");
+                    LOG( LOG_LEVEL_INFO, "starting Xvnc session...");
                     display = session_start(data, SESMAN_SESSION_TYPE_XVNC, c, s);
                 }
                 else if (SCP_SESSION_TYPE_XRDP == s->type)
                 {
-                    log_message(LOG_LEVEL_INFO, "starting X11rdp session...");
+                    LOG(LOG_LEVEL_INFO, "starting X11rdp session...");
                     display = session_start(data, SESMAN_SESSION_TYPE_XRDP, c, s);
                 }
                 else if (SCP_SESSION_TYPE_XORG == s->type)
                 {
                     /* type is SCP_SESSION_TYPE_XORG */
-                    log_message(LOG_LEVEL_INFO, "starting Xorg session...");
+                    LOG(LOG_LEVEL_INFO, "starting Xorg session...");
                     display = session_start(data, SESMAN_SESSION_TYPE_XORG, c, s);
                 }
                 /* if the session started up ok, auth_end will be called on
