@@ -55,14 +55,14 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     retries = g_cfg->sec.login_retry;
     current_try = retries;
 
-    data = auth_userpass(s->username, s->password,NULL);
-    /*LOG_DBG("user: %s\npass: %s", s->username, s->password);*/
+    data = auth_userpass(s->username, s->password, NULL);
+    /*LOG_DEVEL(LOG_LEVEL_DEBUG, "user: %s\npass: %s", s->username, s->password);*/
 
     while ((!data) && ((retries == 0) || (current_try > 0)))
     {
-        LOG_DBG("data %d - retry %d - currenttry %d - expr %d",
-                data, retries, current_try,
-                ((!data) && ((retries == 0) || (current_try > 0))));
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "data %ld - retry %d - currenttry %d - expr %d",
+                  data, retries, current_try,
+                  ((!data) && ((retries == 0) || (current_try > 0))));
 
         e = scp_v1s_request_password(c, s, "Wrong username and/or password");
 
@@ -70,7 +70,7 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
         {
             case SCP_SERVER_STATE_OK:
                 /* all ok, we got new username and password */
-                data = auth_userpass(s->username, s->password,NULL);
+                data = auth_userpass(s->username, s->password, NULL);
 
                 /* one try less */
                 if (current_try > 0)
@@ -90,8 +90,8 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     if (!data)
     {
         scp_v1s_deny_connection(c, "Login failed");
-        log_message( LOG_LEVEL_INFO,
-                     "Login failed for user %s. Connection terminated", s->username);
+        LOG( LOG_LEVEL_INFO,
+             "Login failed for user %s. Connection terminated", s->username);
         return;
     }
 
@@ -99,8 +99,8 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     if (0 == access_login_allowed(s->username))
     {
         scp_v1s_deny_connection(c, "Access to Terminal Server not allowed.");
-        log_message(LOG_LEVEL_INFO,
-                    "User %s not allowed on TS. Connection terminated", s->username);
+        LOG(LOG_LEVEL_INFO,
+            "User %s not allowed on TS. Connection terminated", s->username);
         return;
     }
 
@@ -112,31 +112,31 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
     if (scount == 0)
     {
         /* no disconnected sessions - start a new one */
-        log_message(LOG_LEVEL_DEBUG, "No disconnected sessions for this user "
-                    "- we create a new one");
+        LOG(LOG_LEVEL_DEBUG, "No disconnected sessions for this user "
+            "- we create a new one");
 
         if (0 != s->client_ip)
         {
-            log_message(LOG_LEVEL_INFO, "++ created session (access granted): username %s, ip %s", s->username, s->client_ip);
+            LOG(LOG_LEVEL_INFO, "++ created session (access granted): username %s, ip %s", s->username, s->client_ip);
         }
         else
         {
-            log_message(LOG_LEVEL_INFO, "++ created session (access granted): username %s", s->username);
+            LOG(LOG_LEVEL_INFO, "++ created session (access granted): username %s", s->username);
         }
 
         if (SCP_SESSION_TYPE_XVNC == s->type)
         {
-            log_message(LOG_LEVEL_INFO, "starting Xvnc session...");
+            LOG(LOG_LEVEL_INFO, "starting Xvnc session...");
             display = session_start(data, SESMAN_SESSION_TYPE_XVNC, c, s);
         }
         else if (SCP_SESSION_TYPE_XRDP == s->type)
         {
-            log_message(LOG_LEVEL_INFO, "starting X11rdp session...");
+            LOG(LOG_LEVEL_INFO, "starting X11rdp session...");
             display = session_start(data, SESMAN_SESSION_TYPE_XRDP, c, s);
         }
         else if (SCP_SESSION_TYPE_XORG == s->type)
         {
-            log_message(LOG_LEVEL_INFO, "starting Xorg session...");
+            LOG(LOG_LEVEL_INFO, "starting Xorg session...");
             display = session_start(data, SESMAN_SESSION_TYPE_XORG, c, s);
         }
         /* if the session started up ok, auth_end will be called on
@@ -161,10 +161,10 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
         switch (e)
         {
-                /*case SCP_SERVER_STATE_FORCE_NEW:*/
-                /* we should check for MaxSessions */
+            /*case SCP_SERVER_STATE_FORCE_NEW:*/
+            /* we should check for MaxSessions */
             case SCP_SERVER_STATE_SELECTION_CANCEL:
-                log_message( LOG_LEVEL_INFO, "Connection cancelled after session listing");
+                LOG( LOG_LEVEL_INFO, "Connection cancelled after session listing");
                 break;
             case SCP_SERVER_STATE_OK:
                 /* ok, reconnecting... */
@@ -173,7 +173,7 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
                 if (0 == sitem)
                 {
                     e = scp_v1s_connection_error(c, "Internal error");
-                    log_message(LOG_LEVEL_INFO, "Cannot find session item on the chain");
+                    LOG(LOG_LEVEL_INFO, "Cannot find session item on the chain");
                 }
                 else
                 {
@@ -183,11 +183,11 @@ scp_v1_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
                     if (0 != s->client_ip)
                     {
-                        log_message(LOG_LEVEL_INFO, "++ reconnected session: username %s, display :%d.0, session_pid %d, ip %s", s->username, display, sitem->pid, s->client_ip);
+                        LOG(LOG_LEVEL_INFO, "++ reconnected session: username %s, display :%d.0, session_pid %d, ip %s", s->username, display, sitem->pid, s->client_ip);
                     }
                     else
                     {
-                        log_message(LOG_LEVEL_INFO, "++ reconnected session: username %s, display :%d.0, session_pid %d", s->username, display, sitem->pid);
+                        LOG(LOG_LEVEL_INFO, "++ reconnected session: username %s, display :%d.0, session_pid %d", s->username, display, sitem->pid);
                     }
 
                     g_free(sitem);
@@ -220,27 +220,27 @@ static void parseCommonStates(enum SCP_SERVER_STATES_E e, const char *f)
     switch (e)
     {
         case SCP_SERVER_STATE_VERSION_ERR:
-            LOG_DBG("version error")
+            LOG(LOG_LEVEL_WARNING, "version error");
         case SCP_SERVER_STATE_SIZE_ERR:
             /* an unknown scp version was requested, so we shut down the */
             /* connection (and log the fact)                             */
-            log_message(LOG_LEVEL_WARNING,
-                        "protocol violation. connection closed.");
+            LOG(LOG_LEVEL_WARNING,
+                "protocol violation. connection closed.");
             break;
         case SCP_SERVER_STATE_NETWORK_ERR:
-            log_message(LOG_LEVEL_WARNING, "libscp network error.");
+            LOG(LOG_LEVEL_WARNING, "libscp network error.");
             break;
         case SCP_SERVER_STATE_SEQUENCE_ERR:
-            log_message(LOG_LEVEL_WARNING, "libscp sequence error.");
+            LOG(LOG_LEVEL_WARNING, "libscp sequence error.");
             break;
         case SCP_SERVER_STATE_INTERNAL_ERR:
             /* internal error occurred (eg. malloc() error, ecc.) */
-            log_message(LOG_LEVEL_ERROR, "libscp internal error occurred.");
+            LOG(LOG_LEVEL_ERROR, "libscp internal error occurred.");
             break;
         default:
             /* dummy: scp_v1s_request_password won't generate any other */
             /* error other than the ones before                         */
-            log_message(LOG_LEVEL_ALWAYS, "unknown return from %s", f);
+            LOG(LOG_LEVEL_ALWAYS, "unknown return from %s", f);
             break;
     }
 }
