@@ -17,12 +17,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "arch.h"
 #include "tcp.h"
 #include "libscp.h"
 #include "parse.h"
 #include "log.h"
 #include "libscp.h"
+#include "string_calls.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -33,11 +38,9 @@ char cmnd[257];
 char serv[257];
 char port[257];
 
-struct log_config logging;
-
 void cmndList(struct SCP_CONNECTION *c);
 void cmndKill(struct SCP_CONNECTION *c, struct SCP_SESSION *s);
-void cmndHelp();
+void cmndHelp(void);
 
 int inputSession(struct SCP_SESSION *s);
 unsigned int menuSelect(unsigned int choices);
@@ -52,6 +55,7 @@ int main(int argc, char **argv)
     //int sel;
     int sock;
     char *pwd;
+    struct log_config *logging;
 
     user[0] = '\0';
     pass[0] = '\0';
@@ -59,11 +63,9 @@ int main(int argc, char **argv)
     serv[0] = '\0';
     port[0] = '\0';
 
-    logging.program_name = "sesadmin";
-    logging.log_file = g_strdup("xrdp-sesadmin.log");
-    logging.log_level = LOG_LEVEL_DEBUG;
-    logging.enable_syslog = 0;
-    log_start_from_param(&logging);
+    logging = log_config_init_for_console(LOG_LEVEL_INFO, NULL);
+    log_start_from_param(logging);
+    log_config_free(logging);
 
     for (idx = 0; idx < argc; idx++)
     {
@@ -129,18 +131,18 @@ int main(int argc, char **argv)
     sock = g_tcp_socket();
     if (sock < 0)
     {
-        LOG_DBG("Socket open error, g_tcp_socket() failed");
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "Socket open error, g_tcp_socket() failed");
         return 1;
     }
 
     s = scp_session_create();
     c = scp_connection_create(sock);
 
-    LOG_DBG("Connecting to %s:%s with user %s (%s)", serv, port, user, pass);
+    LOG_DEVEL(LOG_LEVEL_DEBUG, "Connecting to %s:%s with user %s (%s)", serv, port, user, pass);
 
     if (0 != g_tcp_connect(sock, serv, port))
     {
-        LOG_DBG("g_tcp_connect() error");
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "g_tcp_connect() error");
         return 1;
     }
 
@@ -153,7 +155,7 @@ int main(int argc, char **argv)
 
     if (SCP_CLIENT_STATE_OK != e)
     {
-        LOG_DBG("libscp error connecting: %s %d", s->errstr, (int)e);
+        LOG_DEVEL(LOG_LEVEL_DEBUG, "libscp error connecting: %s %d", s->errstr, (int)e);
     }
 
     if (0 == g_strncmp(cmnd, "list", 5))
@@ -173,7 +175,7 @@ int main(int argc, char **argv)
     return 0;
 }
 
-void cmndHelp()
+void cmndHelp(void)
 {
     fprintf(stderr, "sesadmin - a console sesman administration tool\n");
     fprintf(stderr, "syntax: sesadmin [] COMMAND [OPTIONS]\n\n");
@@ -187,6 +189,20 @@ void cmndHelp()
     fprintf(stderr, "               kill:<sid>\n");
 }
 
+static void
+print_session(const struct SCP_DISCONNECTED_SESSION *s)
+{
+    printf("Session ID: %d\n", s->SID);
+    printf("\tSession type: %d\n", s->type);
+    printf("\tScreen size: %dx%d, color depth %d\n",
+           s->width, s->height, s->bpp);
+    printf("\tIdle time: %d day(s) %d hour(s) %d minute(s)\n",
+           s->idle_days, s->idle_hours, s->idle_minutes);
+    printf("\tConnected: %04d/%02d/%02d %02d:%02d\n",
+           s->conn_year, s->conn_month, s->conn_day, s->conn_hour,
+           s->conn_minute);
+}
+
 void cmndList(struct SCP_CONNECTION *c)
 {
     struct SCP_DISCONNECTED_SESSION *dsl;
@@ -196,14 +212,17 @@ void cmndList(struct SCP_CONNECTION *c)
 
     e = scp_v1c_mng_get_session_list(c, &scnt, &dsl);
 
-    if ((SCP_CLIENT_STATE_LIST_OK == e) && (scnt > 0))
+    if (e != SCP_CLIENT_STATE_LIST_OK)
+    {
+        printf("Error getting session list.\n");
+        return;
+    }
+
+    if (scnt > 0)
     {
         for (idx = 0; idx < scnt; idx++)
         {
-            printf("%d\t%d\t%dx%dx%d\t%d-%d-%d\t%04d/%02d/%02d@%02d:%02d\n", \
-                   (dsl[idx]).SID, (dsl[idx]).type, (dsl[idx]).width, (dsl[idx]).height, (dsl[idx]).bpp, \
-                   (dsl[idx]).idle_days, (dsl[idx]).idle_hours, (dsl[idx]).idle_minutes, \
-                   (dsl[idx]).conn_year, (dsl[idx]).conn_month, (dsl[idx]).conn_day, (dsl[idx]).conn_hour, (dsl[idx]).conn_minute);
+            print_session(&dsl[idx]);
         }
     }
     else
@@ -211,10 +230,7 @@ void cmndList(struct SCP_CONNECTION *c)
         printf("No sessions.\n");
     }
 
-    if (0 != dsl)
-    {
-        g_free(dsl);
-    }
+    g_free(dsl);
 }
 
 void cmndKill(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
