@@ -24,16 +24,24 @@
  *
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "arch.h"
 #include "os_calls.h"
+#include "string_calls.h"
 
 #include <stdio.h>
 #include <security/pam_appl.h>
 
+/* Defines the maximum size of a username or password. With pam there is no real limit */
+#define MAX_BUF 8192
+
 struct t_user_pass
 {
-    char user[256];
-    char pass[256];
+    char user[MAX_BUF];
+    char pass[MAX_BUF];
 };
 
 struct t_auth_info
@@ -46,7 +54,7 @@ struct t_auth_info
 };
 
 /******************************************************************************/
-static int DEFAULT_CC
+static int
 verify_pam_conv(int num_msg, const struct pam_message **msg,
                 struct pam_response **resp, void *appdata_ptr)
 {
@@ -70,6 +78,9 @@ verify_pam_conv(int num_msg, const struct pam_message **msg,
                 reply[i].resp = g_strdup(user_pass->pass);
                 reply[i].resp_retcode = PAM_SUCCESS;
                 break;
+            case PAM_TEXT_INFO:
+                g_memset(&reply[i], 0, sizeof(struct pam_response));
+                break;
             default:
                 g_printf("unknown in verify_pam_conv\r\n");
                 g_free(reply);
@@ -82,12 +93,13 @@ verify_pam_conv(int num_msg, const struct pam_message **msg,
 }
 
 /******************************************************************************/
-static void DEFAULT_CC
+static void
 get_service_name(char *service_name)
 {
     service_name[0] = 0;
 
-    if (g_file_exist("/etc/pam.d/xrdp-sesman"))
+    if (g_file_exist("/etc/pam.d/xrdp-sesman") ||
+            g_file_exist(XRDP_SYSCONF_PATH "/pam.d/xrdp-sesman"))
     {
         g_strncpy(service_name, "xrdp-sesman", 255);
     }
@@ -101,7 +113,7 @@ get_service_name(char *service_name)
 /* returns long, zero is no go
  Stores the detailed error code in the errorcode variable*/
 
-long DEFAULT_CC
+long
 auth_userpass(const char *user, const char *pass, int *errorcode)
 {
     int error;
@@ -110,8 +122,8 @@ auth_userpass(const char *user, const char *pass, int *errorcode)
 
     get_service_name(service_name);
     auth_info = g_new0(struct t_auth_info, 1);
-    g_strncpy(auth_info->user_pass.user, user, 255);
-    g_strncpy(auth_info->user_pass.pass, pass, 255);
+    g_strncpy(auth_info->user_pass.user, user, MAX_BUF - 1);
+    g_strncpy(auth_info->user_pass.pass, pass, MAX_BUF - 1);
     auth_info->pamc.conv = &verify_pam_conv;
     auth_info->pamc.appdata_ptr = &(auth_info->user_pass);
     error = pam_start(service_name, 0, &(auth_info->pamc), &(auth_info->ph));
@@ -175,7 +187,7 @@ auth_userpass(const char *user, const char *pass, int *errorcode)
 
 /******************************************************************************/
 /* returns error */
-int DEFAULT_CC
+int
 auth_start_session(long in_val, int in_display)
 {
     struct t_auth_info *auth_info;
@@ -216,7 +228,7 @@ auth_start_session(long in_val, int in_display)
 
 /******************************************************************************/
 /* returns error */
-int DEFAULT_CC
+int
 auth_stop_session(long in_val)
 {
     struct t_auth_info *auth_info;
@@ -237,7 +249,7 @@ auth_stop_session(long in_val)
 /******************************************************************************/
 /* returns error */
 /* cleanup */
-int DEFAULT_CC
+int
 auth_end(long in_val)
 {
     struct t_auth_info *auth_info;
@@ -270,7 +282,7 @@ auth_end(long in_val)
 /******************************************************************************/
 /* returns error */
 /* set any pam env vars */
-int DEFAULT_CC
+int
 auth_set_env(long in_val)
 {
     struct t_auth_info *auth_info;
