@@ -25,6 +25,7 @@
 #include "base64.h"
 #include "xrdp.h"
 #include "log.h"
+#include "string_calls.h"
 
 #define ASK "ask"
 #define ASK_LEN g_strlen(ASK)
@@ -559,8 +560,8 @@ xrdp_wm_login_fill_in_combo(struct xrdp_wm *self, struct xrdp_bitmap *b)
     char *q;
     char *r;
     char name[256];
-    char cfg_file[256];
     struct xrdp_mod_data *mod_data;
+    const char *xrdp_ini = self->session->xrdp_ini;
 
     sections = list_create();
     sections->auto_free = 1;
@@ -568,12 +569,12 @@ xrdp_wm_login_fill_in_combo(struct xrdp_wm *self, struct xrdp_bitmap *b)
     section_names->auto_free = 1;
     section_values = list_create();
     section_values->auto_free = 1;
-    g_snprintf(cfg_file, 255, "%s/xrdp.ini", XRDP_CFG_PATH);
-    fd = g_file_open(cfg_file); /* xrdp.ini */
+    fd = g_file_open(xrdp_ini);
 
     if (fd < 0)
     {
-        log_message(LOG_LEVEL_ERROR, "Could not read xrdp ini file %s", cfg_file);
+        log_message(LOG_LEVEL_ERROR, "Could not read xrdp ini file %s",
+                    xrdp_ini);
         list_delete(sections);
         list_delete(section_names);
         list_delete(section_values);
@@ -589,7 +590,8 @@ xrdp_wm_login_fill_in_combo(struct xrdp_wm *self, struct xrdp_bitmap *b)
 
         if ((g_strncasecmp(p, "globals", 255) == 0)
                 || (g_strncasecmp(p, "channels", 255) == 0)
-                || (g_strncasecmp(p, "Logging", 255) == 0))
+                || (g_strncasecmp(p, "Logging", 255) == 0)
+                || (g_strncasecmp(p, "LoggingPerLogger", 255) == 0))
         {
         }
         else
@@ -829,10 +831,14 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
 /**
  * Load configuration from xrdp.ini file
  *
+ * @param config XRDP configuration to initialise
+ * @param xrdp_ini Path to xrdp.ini
+ * @param bpp bits-per-pixel for this connection
+ *
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 int
-load_xrdp_config(struct xrdp_config *config, int bpp)
+load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
 {
     struct xrdp_cfg_globals  *globals;
 
@@ -841,7 +847,6 @@ load_xrdp_config(struct xrdp_config *config, int bpp)
 
     char *n;
     char *v;
-    char  buf[256];
     int   fd;
     int   i;
 
@@ -873,11 +878,10 @@ load_xrdp_config(struct xrdp_config *config, int bpp)
     globals->ls_btn_cancel_height = 30;
 
     /* open xrdp.ini file */
-    g_snprintf(buf, 255, "%s/xrdp.ini", XRDP_CFG_PATH);
-    if ((fd = g_file_open(buf)) < 0)
+    if ((fd = g_file_open(xrdp_ini)) < 0)
     {
         log_message(LOG_LEVEL_ERROR,"load_config: Could not read "
-                    "xrdp.ini file %s", buf);
+                    "xrdp.ini file %s", xrdp_ini);
         return -1;
 
     }
@@ -893,7 +897,7 @@ load_xrdp_config(struct xrdp_config *config, int bpp)
         list_delete(values);
         g_file_close(fd);
         log_message(LOG_LEVEL_ERROR,"load_config: Could not read globals "
-                    "section from xrdp.ini file %s", buf);
+                    "section from xrdp.ini file %s", xrdp_ini);
         return -1;
     }
 
@@ -1001,6 +1005,11 @@ load_xrdp_config(struct xrdp_config *config, int bpp)
         else if (g_strncmp(n, "allow_multimon", 64) == 0)
             globals->allow_multimon = g_text2bool(v);
 
+        else if (g_strncmp(n, "enable_token_login", 64) == 0) {
+            log_message(LOG_LEVEL_DEBUG, "Token login detection enabled x");
+            globals->enable_token_login = g_text2bool(v);
+        }
+
         /* login screen values */
         else if (g_strncmp(n, "ls_top_window_bg_color", 64) == 0)
             globals->ls_top_window_bg_color = HCOLOR(bpp, xrdp_wm_htoi(v));
@@ -1109,12 +1118,13 @@ load_xrdp_config(struct xrdp_config *config, int bpp)
     g_writeln("new_cursors:             %d", globals->new_cursors);
     g_writeln("nego_sec_layer:          %d", globals->nego_sec_layer);
     g_writeln("allow_multimon:          %d", globals->allow_multimon);
+    g_writeln("enable_token_login:      %d", globals->enable_token_login)
 
     g_writeln("ls_top_window_bg_color:  %x", globals->ls_top_window_bg_color);
     g_writeln("ls_width:                %d", globals->ls_width);
     g_writeln("ls_height:               %d", globals->ls_height);
     g_writeln("ls_bg_color:             %x", globals->ls_bg_color);
-    g_writeln("ls_title:            %s", globals->ls_title);
+    g_writeln("ls_title:                %s", globals->ls_title);
     g_writeln("ls_logo_filename:        %s", globals->ls_logo_filename);
     g_writeln("ls_logo_x_pos:           %d", globals->ls_logo_x_pos);
     g_writeln("ls_logo_y_pos:           %d", globals->ls_logo_y_pos);
