@@ -18,13 +18,17 @@
  * main rdp process
  */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include "xrdp.h"
 
 static int g_session_id = 0;
 
 /*****************************************************************************/
 /* always called from xrdp_listen thread */
-struct xrdp_process *APP_CC
+struct xrdp_process *
 xrdp_process_create(struct xrdp_listen *owner, tbus done_event)
 {
     struct xrdp_process *self;
@@ -44,7 +48,7 @@ xrdp_process_create(struct xrdp_listen *owner, tbus done_event)
 }
 
 /*****************************************************************************/
-void APP_CC
+void
 xrdp_process_delete(struct xrdp_process *self)
 {
     if (self == 0)
@@ -60,7 +64,7 @@ xrdp_process_delete(struct xrdp_process *self)
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_process_loop(struct xrdp_process *self, struct stream *s)
 {
     int rv;
@@ -87,14 +91,14 @@ xrdp_process_loop(struct xrdp_process *self, struct stream *s)
 /*****************************************************************************/
 /* returns boolean */
 /* this is so libxrdp.so can known when to quit looping */
-static int DEFAULT_CC
+static int
 xrdp_is_term(void)
 {
     return g_is_term();
 }
 
 /*****************************************************************************/
-static int APP_CC
+static int
 xrdp_process_mod_end(struct xrdp_process *self)
 {
     if (self->wm != 0)
@@ -115,7 +119,7 @@ xrdp_process_mod_end(struct xrdp_process *self)
 }
 
 /*****************************************************************************/
-static int DEFAULT_CC
+static int
 xrdp_process_data_in(struct trans *self)
 {
     struct xrdp_process *pro;
@@ -212,7 +216,7 @@ xrdp_process_data_in(struct trans *self)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 xrdp_process_main_loop(struct xrdp_process *self)
 {
     int robjs_count;
@@ -231,7 +235,8 @@ xrdp_process_main_loop(struct xrdp_process *self)
     self->server_trans->trans_data_in = xrdp_process_data_in;
     self->server_trans->callback_data = self;
     init_stream(self->server_trans->in_s, 8192 * 4);
-    self->session = libxrdp_init((tbus)self, self->server_trans);
+    self->session = libxrdp_init((tbus)self, self->server_trans,
+                                 self->lis_layer->startup_params->xrdp_ini);
     self->server_trans->si = &(self->session->si);
     self->server_trans->my_source = XRDP_SOURCE_CLIENT;
     /* this callback function is in xrdp_wm.c */
