@@ -25,6 +25,7 @@
 #include "xup.h"
 #include "log.h"
 #include "trans.h"
+#include "string_calls.h"
 
 #define LOG_LEVEL 1
 #define LLOG(_level, _args) \
@@ -149,7 +150,6 @@ lib_mod_connect(struct mod *mod)
     int use_uds;
     struct stream *s;
     char con_port[256];
-    struct source_info *si;
 
     LIB_DEBUG(mod, "in lib_mod_connect");
 
@@ -203,8 +203,7 @@ lib_mod_connect(struct mod *mod)
         }
     }
 
-    si = (struct source_info *) (mod->si);
-    mod->trans->si = si;
+    mod->trans->si = mod->si;
     mod->trans->my_source = XRDP_SOURCE_MOD;
 
     while (1)
@@ -1546,11 +1545,11 @@ lib_mod_set_param(struct mod *mod, const char *name, const char *value)
 {
     if (g_strcasecmp(name, "username") == 0)
     {
-        g_strncpy(mod->username, value, 255);
+        g_strncpy(mod->username, value, INFO_CLIENT_MAX_CB_LEN-1);
     }
     else if (g_strcasecmp(name, "password") == 0)
     {
-        g_strncpy(mod->password, value, 255);
+        g_strncpy(mod->password, value, INFO_CLIENT_MAX_CB_LEN-1);
     }
     else if (g_strcasecmp(name, "ip") == 0)
     {
